﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Http;

using FootballPredictions.DataHandling;
using FootballPredictions.Evaluation;
using FootballPredictions.JAGS;
using FootballPredictions.JAGS.DixonColesDynamic;
using FootballPredictions.Evaluation.Betting;
using FootballPredictions.JAGS.RueSalvesen;
using FootballPredictions.DataHandling.Plots;
using FootballPredictions.Evaluation.Betting.BettingStrategies;

namespace FootballPredictions
{
    /// <summary>
    /// This is our main class, where the simulations are started. 
    /// It should be extended to create a GUI, but this have not been done.
    /// </summary>
    public static class WebApiConfig
    {
        public static void Register(HttpConfiguration config)
        {
            config.Routes.MapHttpRoute(
                name: "DefaultApi",
                routeTemplate: "api/{controller}/{id}",
                defaults: new { id = RouteParameter.Optional }
            );

            // Uncomment the following line of code to enable query support for actions with an IQueryable or IQueryable<T> return type.
            // To avoid processing unexpected or malicious queries, use the validation settings on QueryableAttribute to validate incoming queries.
            // For more information, visit http://go.microsoft.com/fwlink/?LinkId=279712.
            //config.EnableQuerySupport();

            // To disable tracing in your application, please comment out or remove the following line of code
            // For more information, refer to: http://www.asp.net/web-api
            config.EnableSystemDiagnosticsTracing();

            NHibernateConfig NHConfig = new NHibernateConfig();

            SeasonSimulator Simulation;

            switch (Constants.SIMULATION_MODEL)
            {
                case SimulationModel.RueSalvesen:
                    {
                        Simulation = new RueSalvesenSimulation();
                        break;
                    }
                case SimulationModel.GoalScaledRueSalvesen:
                    {
                        Simulation = new GoalScaledRueSalvesenSimulation();
                        break;
                    }
                case SimulationModel.DixonColes:
                    {
                        Simulation = new DixonColesDynamicSimulation();
                        break;
                    }
                case SimulationModel.DixonColesIndividual:
                    {
                        Simulation = new IndividualDixonColesDynamicSimulation();
                        break;
                    }
                case SimulationModel.DataIntensiveScaledAttackAndDefence:
                    {
                        Simulation = new DataIntensiveScaledAttackAndDefenceSimulation();
                        break;
                    }
                case SimulationModel.DataIntensiveScaledAttack:
                    {
                        Simulation = new DataIntensiveScaledAttackSimulation();
                        break;
                    }
                default:
                    {
                        if (Constants.SIMULATE_BETS)
                        {
                            Betting bet;
                            switch (Constants.BETTING_MODEL)
                            {
                                case BettingModel.FixedBet:
                                    {
                                        bet = new FixedBet(Constants.SIMULATION_ID_TO_PLOT, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                                        break;
                                    }
                                case BettingModel.FixedBetNew:
                                    {
                                        bet = new FixedBetNew(Constants.SIMULATION_ID_TO_PLOT, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                                        break;
                                    }
                                case BettingModel.FixedReturn:
                                    {
                                        bet = new FixedReturn(Constants.SIMULATION_ID_TO_PLOT, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                                        break;
                                    }
                                case BettingModel.ProbabilityBet:
                                    {
                                        bet = new ProbabilityBet(Constants.SIMULATION_ID_TO_PLOT, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                                        break;
                                    }
                                case BettingModel.Wallpapering:
                                    {
                                        bet = new Wallpapering(Constants.SIMULATION_ID_TO_PLOT, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                                        break;
                                    }
                                case BettingModel.Wallpapering2:
                                    {
                                        bet = new Wallpapering2(Constants.SIMULATION_ID_TO_PLOT, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                                        break;
                                    }
                                case BettingModel.Kelly:
                                    {
                                        bet = new Kelly(Constants.SIMULATION_ID_TO_PLOT, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                                        break;
                                    }
                            }
                        }
                        if (Constants.GENERATE_RPS)
                        {
                            new GenerateRPS(Constants.SIMULATION_ID_TO_PLOT);
                        }
                        break;
                    }
            }
            if (Constants.AGGREGATE_SIMULATION)
            {
                new AggregatedSimulator(Constants.SIMULATION_IDS_AGGREGATE);
            }
            if (Constants.COMBINE_SIMULATIONS)
            {
                CombineSimulations combiner = new CombineSimulations(Constants.SIMULATION_IDS_TO_COMBINE);
                combiner.Combine();
            }
            if (Constants.GENERATE_AUTOCORRELATION_PLOT)
            {
                var autocorr = new Autocorrelation(Constants.MONITOR_VALUE, Constants.TEAM_NUMBER, Constants.ROUND_NUMBER, Constants.TIME_SLICES);
                autocorr.CreatePlot();
            }
            if (Constants.PLOT_MONITOR_VALUES)
            {
                var monitorPlot = new PlotMonitorValues(Constants.MONITOR_VALUE, Constants.TEAM_NUMBER, Constants.ROUND_NUMBER);
                monitorPlot.CreatePlot();
            }
            if (Constants.PLOT_TEAM_STRENGTH)
            {
                PlotTeamStrength pts = new PlotTeamStrength()
                {
                    StageId = Constants.STAGE_ID,
                    TeamName = Constants.TEAM_NAME,
                    SimulationId = Constants.SIMULATION_ID_TO_PLOT
                };
                pts.Plot();
            }
        }
    }
}
