﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace FootballPredictions
{
    /**
     * The constants that should be changed when running the program, 
     * like file paths etc.
     **/
    public class Constants
    {
        //Paths for text files, database and JAGS
        //public static string CURRENT_USER = "C:/Johanne/tdt4900repo/tdt4201fdp/tdt4201fdp"; //Johanne lesesal
        public static string CURRENT_USER = "C:/Users/Johanne/Documents/Johanne/NTNU/Haust2013/TDT4201Prosjekt"; //Johanne bærbar
        //public static string CURRENT_USER = "C:/Users/lokketangen/Skole/Prosjekt/tdt4201fdp"; //Marius bærbar
        public static string TEXT_FILES = CURRENT_USER + "/code master/FootballPredictions/FootballPredictions/TextFiles/";
        public static string DATABASE = CURRENT_USER + "/code master/FootballPredictions/FootballPredictions/App_Data/Database.sdf";
        public static string BIN_PATH = CURRENT_USER + "/code master/FootballPredictions/FootballPredictions/Dependencies/JAGS/WIN/bin/";
        
        
        //----------------------------------------------------------
        // Current stage
        // 5476 = 11/12, 6531 = 12/13, 7794 = 13/14
        public static int STAGE_ID = 7794;
        public static string COUNTRY = "England";
        public static string LEAGUE = "Premier League";
        public static string YEAR = "2013/2014";

        // Database stuff ---------------------------------------------------------
        // If a new table should be added to the database NB: This wipes the current content, and adds everything to the database. Runtime: 10-20 hrs.
        public static bool ADD_NEW_TABLE = false;
        // If the database should be updated during the execution of the program (except during simulations) (e.g new odds or new matches)
        public static bool UPDATE_DATABASE = false;

        // Set up of simulation ----------------------------------------------------
        public static SimulationModel SIMULATION_MODEL = SimulationModel.NoSimulation;

        // If the rest of the season should be simulated with the given training set.
        public static int SIZE_OF_TRAINING_SET = 370;
        public static bool SEASON_SIMULATION = false; // false means that the next match is simulated with all the available information (recommended)

        // If the matches in the given range should be simulated
        public static int SIMULATION_START_MATCH = 190;
        public static int SIMULATION_END_MATCH = 380;
        public static int THINNING = 20; // SAMPLE_COUNT/THINNING = Integer. most safe :)
        
        // MCMCParameters
        public static int BURN_IN = 500;
        public static int SAMPLE_COUNT = 40000;
        public static int CHAINS = 1; // Set this to 1, since the SharpJags package will create identical CODA files. Use Combine simulations after a number of runs with the same model.

        public static int SAMPLES_AFTER_THINNING = (int)Math.Ceiling((double)Constants.SAMPLE_COUNT / (double)Constants.THINNING);

        // Write statistics to file ---------------------------------------------------------

        // If SimulationModel is set to NoSimulation, this is the simulation to create plots from
        public static int SIMULATION_ID_TO_PLOT = 4;

        // Betting
        public static bool SIMULATE_BETS = false;
        public static BettingModel BETTING_MODEL = BettingModel.Wallpapering;
        public static string BOOKMAKER_ID = "BbAv";
        public static double SAFETY_MARGIN = 0.05; // Used in FixedReturn and FixedBetNew
        public static double UNIT = 1.0; // How much to bet
        public static DateTime START_DATE_FOR_PLOTS = new DateTime(2011,04,07);

        // RPS
        public static bool GENERATE_RPS = false;

        // Team strength
        public static bool PLOT_TEAM_STRENGTH = false;
        public static string TEAM_NAME = "Cardiff";


        // Autocorrelation and monitor value plot (created from CODA files in /TextFiles/JAGS)
        public static bool GENERATE_AUTOCORRELATION_PLOT = false; //NB! Set MCMC params and thinning.
        public static bool PLOT_MONITOR_VALUES = false; //NB! Set MCMC params and thinning.
        public static int TEAM_NUMBER = 4;
        public static int ROUND_NUMBER = 31;
        public static string MONITOR_VALUE = "attack";
        public static int TIME_SLICES = 20;

        // Aggregate simulation
        public static bool AGGREGATE_SIMULATION = false;
        public static int[] SIMULATION_IDS_AGGREGATE = new int[] { 1, 2, 3, 5 };

        // Combine simulations
        public static bool COMBINE_SIMULATIONS = false;
        public static int[] SIMULATION_IDS_TO_COMBINE = new int[] { 1, 2, 7 };

        // Used in betting plots
        public static string StringOfConstants()
        {
            var text = "%"
                + "\n% Stage id: " + STAGE_ID
                + "\n% Simulation model: " + SIMULATION_MODEL.ToString()
                + "\n% Season simulation: " + SEASON_SIMULATION
                + "\n% Size of test set: " + SIZE_OF_TRAINING_SET
                + "\n% Simulation start match: " + SIMULATION_START_MATCH
                + "\n% Simulation end match: " + SIMULATION_END_MATCH
                + "\n% Thinning: " + THINNING
                + "\n% Burn in: " + BURN_IN
                + "\n% Sample count: " + SAMPLE_COUNT
                + "\n% Chains: " + CHAINS
                + "\n% Simulate bets: " + SIMULATE_BETS
                + "\n% Betting model: " + BETTING_MODEL.ToString()
                + "\n% Bookamker id: " + BOOKMAKER_ID
                + "\n% Safety margin: " + SAFETY_MARGIN
                + "\n% Unit: " + UNIT
                + "\n% Simulation id to plot: " + SIMULATION_ID_TO_PLOT
                + "\n% Generate RPS: " + GENERATE_RPS
                + "\n% Date: " + DateTime.Now;
            
            return text;
        }
    }

    /// <summary>
    /// Enum for the different simulation models.
    /// NB: Update the switch in WebApiConfig when a new simulation model is added.
    /// </summary>
    public enum SimulationModel
    {
        DixonColes,
        DixonColesIndividual,
        RueSalvesen,
        GoalScaledRueSalvesen,
        DataIntensiveScaledAttack,
        DataIntensiveScaledAttackAndDefence,
        NoSimulation,
    }

    /// <summary>
    /// Enum for the different betting models.
    /// NB: Update the switch in SeasonSimulator AND in WebApiConfig when a new betting model is added.
    /// </summary>
    public enum BettingModel
    {
        FixedBet,
        FixedBetNew,
        FixedReturn,
        ProbabilityBet,
        Wallpapering,
        Wallpapering2,
        Kelly,
    }
}