﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;

namespace FootballPredictions.DataHandling
{
    public class DocumentFormatter
    {
        public DocumentFormatter()
        {

        }

        /// <summary>
        /// Method for formatting a data file from the crawler, \n
        /// removing unnecessary symbols and making the documents more readable.
        /// Returns the path of a new .txt file containing the formatted string.
        /// </summary>
        /// <returns>The path of a new .txt file containing the formatted version of document</returns>
        public string FormatMatchDocument(FileInfo document)
        {
            var savePath = Constants.TEXT_FILES + Constants.STAGE_ID + "/FormattedFiles/" + document.Name;
            
            if (File.Exists(savePath))
            {
                return savePath;
            }

            System.IO.StreamReader reader = new StreamReader(document.FullName);
            string line;
            IList<string> lines = new List<string>();

            while ((line = reader.ReadLine()) != null)
            {
                if (line.StartsWith("var initialData"))
                {
                    line = FormatFirstLine(line);
                }
                else if (line.StartsWith(", [["))
                {
                    line = FormatTeamAndKeeperLine(line);
                }

                else if (line.Equals("]"))
                {
                    var nextLine = reader.ReadLine();
                    if (nextLine.StartsWith(",[") && nextLine.Length > 6)
                    {
                        line = FormatTeamAndKeeperLine(nextLine);
                    }
                    else
                    {
                        continue;
                    }

                }

                else if (line.StartsWith(",[") && line.Length > 6)
                {
                    line = FormatPlayerLine(line);
                }
                else if (!(line.StartsWith("home:") || line.StartsWith("away:")))
                {
                    continue;
                }
                lines.Add(line);
            }

//            using (File.Create(savePath)) { }
            using (System.IO.StreamWriter file = new System.IO.StreamWriter(savePath))
            {
                foreach (string l in lines)
                {
                        file.WriteLine(l);
                }
            }

            return savePath;
        }

        private string FormatPlayerLine(string line)
        {
            var newLine = "Player";
            line = line.Replace("[", "");
            line = line.Replace("]", "");
            line = line.Replace("'", "");

            return newLine+line;
        }

        private string FormatTeamAndKeeperLine(string line)
        {
            line = line.Substring(2);
            line = line.Replace("'", "");

            var lineSplit = line.IndexOf("]]]],[[");
            var teamLine = line.Substring(0, lineSplit + 1);
            var keeperLine = line.Substring(lineSplit + 7);

            teamLine = teamLine.Replace("[", "");
            teamLine = teamLine.Replace("]", "");
            keeperLine = keeperLine.Replace("[", "");
            keeperLine = keeperLine.Replace("]", "");

            return "Team," + teamLine + "\nPlayer," + keeperLine;
        }

        private string FormatFirstLine(string line)
        {
            line = Regex.Replace(line, "[^0-9,/:' ]", "").Trim();
            string[] lineTable = line.Split(',');
            line = "MatchInfo";
            for (int i = 0; i < lineTable.Length; i++) 
            {
                if (i == 0 || i == 1 || i == 4 || i == lineTable.Length - 1)
                {
                    line += ("," + lineTable[i]);
                }
            }

            return line;
        }
    }
}