﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;

using FootballPredictions.Evaluation;
using FootballPredictions.JAGS;
using FootballPredictions.Mappings;
using System.Globalization;

namespace FootballPredictions.DataHandling.Plots
{
    /// <summary>
    /// Calculates the RPS of a simulation.
    /// </summary>
    public class GenerateRPS
    {
        List<Prediction> Predictions = new List<Prediction>();
        Simulation Simulation;

        public GenerateRPS(int i)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                Simulation = session.Get<Simulation>(i);
                Predictions = GetPredictionList(Simulation);
            }
            WriteRPSToTex(i);
        }

        public GenerateRPS(Simulation simulation)
        {
            Simulation = simulation;
            Predictions = GetPredictionList(simulation);
            WriteRPSToTex(Simulation.SimulationId);
        }

        private List<Prediction> GetPredictionList(Simulation simulation)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {

                    var predictions = from p in session.Query<Prediction>().ToList()
                                      where p.Simulation == simulation
                                      select p;
                    var predictionList = new List<Prediction>();
                    foreach (var prediction in predictions)
                    {
                        predictionList.Add(prediction);
                    }
                    return predictionList;
                }
            }
        }

        /// <summary>
        /// Generates a plot of RPS for a given simulation and writes it to a .tex file.
        /// </summary>
        public void WriteRPSToTex(int simulationId)
        {
            using (System.IO.StreamWriter file = new System.IO.StreamWriter(
                Constants.TEXT_FILES + "JAGS/RPS/"
                + Constants.SAMPLE_COUNT + "_"
                + Constants.BURN_IN + "_"
                + Constants.CHAINS + "_"
                + simulationId + "_"
                + "output.tex"))
            {
                var prefix = "\\begin{figure} \n"
                            + "\\centering \n"
                            + "\\begin{tikzpicture} \n"
                            + "\\begin{axis}\n"
                            + "[legend pos=outer north east, \n"
                            + "xlabel=Match, ylabel=RPS]";

                var suffix = "\\legend{SimulationId" + simulationId + ",Avg " + simulationId + ",} \n"
                            + "\\end{axis} \n"
                            + "\\end{tikzpicture}\\newline"
                            + "\\caption{<ENTER Figure Text>}% \n"
                            + "\\label{pgfplots:<ENTER Caption>}% \n"
                            + "\\end{figure}";

                file.WriteLine(prefix);

                file.WriteLine("\\addplot[color=red,mark=x] coordinates {");
                var Rps = new RPS(simulationId);
                var rps = Rps.RPSs;
                var counter = 0;
                if (Constants.SEASON_SIMULATION)
                {
                    foreach (var d in rps)
                    {
                        if (counter > Constants.SIZE_OF_TRAINING_SET)
                        {
                            file.WriteLine("(" + (counter - Constants.SIZE_OF_TRAINING_SET) + "," + d.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")");
                        }
                        counter++;
                    }
                    file.WriteLine("};");

                    file.WriteLine("\\addplot[color=yellow,mark=x] coordinates {");
                    counter = 0;

                    var avg = Rps.AvgRps;
                    file.WriteLine("(" + (0) + "," + avg.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")");
                    file.WriteLine("(" + (rps.Count - Constants.SIZE_OF_TRAINING_SET) + "," + avg.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")");
                    file.WriteLine("};");
                }
                else
                {
                    foreach (var d in rps)
                    {
                        file.WriteLine("(" + counter + "," + d.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")");
                        counter++;
                    }
                    file.WriteLine("};");

                    file.WriteLine("\\addplot[color=yellow,mark=x] coordinates {");
                    counter = 0;

                    var avg = Rps.AvgRps;
                    file.WriteLine("(" + (0) + "," + avg.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")");
                    file.WriteLine("(" + rps.Count + "," + avg.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")");
                    file.WriteLine("};");
                }
                
                file.WriteLine(suffix);
                file.Write(Constants.StringOfConstants());
            }
        }
    }
}