﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;
using FootballPredictions.Mappings;
using System.IO;
using System.Globalization;

namespace FootballPredictions.DataHandling.Plots
{
    /// <summary>
    /// Plots the team strength of a given team.
    /// </summary>
    public class PlotTeamStrength
    {
        public string TeamName { get; set; }
        public int StageId { get; set; }
        public int SimulationId { get; set; }
        private Simulation Sim;

        public void Plot()
        {
            List<TeamStrength> teamStrengths = GetTeamStrengths();
            var coordinates = GeneratePlotCoordinates(teamStrengths);
            var prefix = "\\begin{figure}\n"
                       + "\\centering\n"
                       + "\\begin{tikzpicture}\n"
                       + "\\begin{axis} \n"
                       + "[xlabel=Round] \n";
            var suffix = "\\legend{Attack strength, Defence strength}\n"
                       + "\\end{axis}\n"
                       + "\\end{tikzpicture}\n"
                       + "\\label{fig:<enter label here>}\n"
                       + "\\caption{Team strength for " + TeamName + " in the " + StageId + " season}\n"
                       + "\\end{figure}\n";
            

            using (StreamWriter file = new StreamWriter(Constants.TEXT_FILES + "/JAGS/" + "teamStrength_" + TeamName + "_" + StageId + "_" + Sim.Name + ".tex"))
            {
                file.Write(prefix+coordinates+suffix);
            }
        }

        /// <summary>
        /// Gets and sorts team strengths for a given team.
        /// </summary>
        private List<TeamStrength> GetTeamStrengths()
        {
            List<TeamStrength> teamStrengths = new List<TeamStrength>();
            using (var session = NHibernateConfig.OpenSession())
            {
                Sim = session.Get<Simulation>(SimulationId);
                var predictions = from p in session.Query<Prediction>().ToList()
                                  where p.Match.Stage.StageId == StageId && ( p.Match.HomeTeam.Name == ("\'" + TeamName + "\'") || p.Match.AwayTeam.Name == ("\'" + TeamName + "\'") ) && p.Simulation.SimulationId == SimulationId
                                  select p;
                List<Prediction> allPredictions = predictions.ToList();
                allPredictions.OrderBy(p => p.Match.DateAndTime);

                foreach (var p in allPredictions)
                {
                    if(p.Match.HomeTeam.Name == "\'"+TeamName+"\'")
                    {
                        teamStrengths.Add(session.Get<TeamStrength>(p.HomeStrength.TeamStrengthId));
                    }
                    else
                    {
                        teamStrengths.Add(session.Get<TeamStrength>(p.AwayStrength.TeamStrengthId));
                    }
                }
            }
            return teamStrengths;
        }

        private string GeneratePlotCoordinates(List<TeamStrength> teamStrengths)
        {
            var attackPlot = "\\addplot [smooth,color=red] plot coordinates { \n";
            var defencePlot = "\\addplot [smooth,color=blue] plot coordinates { \n";
            var counter = 20;
            foreach (var ts in teamStrengths)
            {
                attackPlot += "(" + counter + "," + ts.Attack.ToString("F3", CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                defencePlot += "(" + counter + "," + ts.Defence.ToString("F3", CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                counter++;
            }
            attackPlot += "};\n";
            defencePlot += "};\n";
            
            return attackPlot + defencePlot;
        }
    }
}