﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;
using NHibernate;
using NHibernate.Linq;

using FootballPredictions.Mappings;
using System.IO;

namespace FootballPredictions.DataHandling.PopulateDatabase
{
    /// <summary>
    /// This adds the odds from Norsk Tipping to the database.
    /// </summary>
    public class PopulateNorskTipping
    {
        public PopulateNorskTipping()
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                // Adds the bookmaker to the database
                using (var tx = session.BeginTransaction())
                {
                    var norskTipping = new Bookmaker()
                    {
                        Name = "Norsk Tipping",
                        BookmakerId = "NT"
                    };
                    session.SaveOrUpdate(norskTipping);
                    tx.Commit();
                }

                PopulateOddsFromNorskTipping(session);
            }
        }

        public void PopulateOddsFromNorskTipping(ISession session)
        {
            FileInfo file = new FileInfo(@"" + Constants.TEXT_FILES + Constants.STAGE_ID + "/Odds/langoddsenOdds.csv");
            StreamReader fileReader = new StreamReader(file.FullName);

            var betLine1 = fileReader.ReadLine();
            betLine1 = fileReader.ReadLine();
            while (betLine1 != null)
            {
                var betLine2 = fileReader.ReadLine();
                var betLine3 = fileReader.ReadLine();
                
                var betLineArray1 = betLine1.Split(';');
                var betLineArray2 = betLine2.Split(';');
                var betLineArray3 = betLine3.Split(';');

                var teamArray = betLineArray1[1].Split('-');
                var homeTeam = teamArray[0].Substring(0, teamArray[0].Length - 1);
                var awayTeam = teamArray[1].Substring(1);

                //Get the correct match from DB
                var matchId = from m in session.Query<Match>().ToList()
                              where m.Stage.StageId == Constants.STAGE_ID
                              && m.HomeTeam.Name.Equals("'" + homeTeam + "'")
                              && m.AwayTeam.Name.Equals("'" + awayTeam + "'")
                              select m;


                var homeOdds = Convert.ToDouble(betLineArray1[3], CultureInfo.GetCultureInfo("nb-NO"));
                var drawOdds = Convert.ToDouble(betLineArray2[3], CultureInfo.GetCultureInfo("nb-NO"));
                var awayOdds = Convert.ToDouble(betLineArray3[3], CultureInfo.GetCultureInfo("nb-NO"));

                using (var tx = session.BeginTransaction())
                {
                    //var existingOdds = from o in session.Query<Odds>().ToList()
                    //                   where o.Match == matchId.First()
                    //                   && o.Bookmaker == session.Get<Bookmaker>("NT")
                    //                   select o;
                    //if (existingOdds.Count() == 0)
                    //{
                        var oddsId = new Odds()
                        {
                            Match = session.Get<Match>(matchId.First().MatchId),
                            Bookmaker = session.Get<Bookmaker>("NT"),
                            HomeOdds = homeOdds,
                            DrawOdds = drawOdds,
                            AwayOdds = awayOdds,
                        };
                        session.Save(oddsId);
                        tx.Commit();
                    //}
                }
                betLine1 = fileReader.ReadLine();
            }

        }

    }
}