﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using NHibernate.Linq;

using FootballPredictions.Mappings;
using System.Globalization;

namespace FootballPredictions.DataHandling.PopulateDatabase
{
    /// <summary>
    /// THe odds are added to the database. The runtime of this class is large.
    /// It can be fixed if we know the format of the odds file, and some hardcoding, but this was not prioritized.
    /// </summary>
    public class PopulateOdds
    {
        public PopulateOdds()
        {
            FileInfo file = new FileInfo(@"" + Constants.TEXT_FILES + Constants.STAGE_ID + "/Odds/odds.csv");

            StreamReader fileReader = new StreamReader(file.FullName);

            using (var session = NHibernateConfig.OpenSession())
            {
                var header = fileReader.ReadLine();
                var headerArray = header.Split(',');

                var betLine = fileReader.ReadLine();
                while (betLine != null)
                {
                    var betLineArray = betLine.Split(',');
                    var homeTeam = betLineArray[2];
                    var awayTeam = betLineArray[3];

                    //Get the correct match from DB
                    var matchId = from m in session.Query<Match>().ToList()
                                  where m.Stage.StageId == Constants.STAGE_ID
                                  && m.HomeTeam.Name.Equals("'" + homeTeam + "'")
                                  && m.AwayTeam.Name.Equals("'" + awayTeam + "'")
                                  select m;

                    var counter = 0;
                    foreach (var h in headerArray)
                    {
                        var charH = h.ToCharArray();

                        var bookmaker = h.Substring(0, h.Length - 1);
                        var b = session.Get<Bookmaker>(bookmaker);
                        if (b != null)
                        {
                            Odds oddsId;
                            using (var tx = session.BeginTransaction())
                            {
                                var existingOdds = from o in session.Query<Odds>().ToList()
                                                   where o.Match == matchId.First()
                                                   && o.Bookmaker == b
                                                   select o;
                                if (existingOdds.Count() == 0)
                                {
                                    oddsId = new Odds()
                                    {
                                        Match = session.Get<Match>(matchId.First().MatchId),
                                        Bookmaker = b,
                                    };
                                    session.Save(oddsId);
                                    tx.Commit();
                                }
                                else
                                {
                                    oddsId = existingOdds.First();
                                }
                            }
                            using (var tx = session.BeginTransaction())
                            {
                                var odds = session.Get<Odds>(oddsId.OddsId);
                                var oddsValue = betLineArray[counter];
                                if (charH.Last().Equals('H'))
                                {
                                    if (oddsValue.Equals(""))
                                    {
                                        odds.HomeOdds = 0.0;
                                    }
                                    else
                                    {
                                        odds.HomeOdds = Convert.ToDouble(betLineArray[counter], CultureInfo.GetCultureInfo("en-GB"));
                                    }
                                }
                                else if (charH.Last().Equals('D'))
                                {
                                    if (oddsValue.Equals(""))
                                    {
                                        odds.DrawOdds = 0.0;
                                    }
                                    else
                                    {
                                        odds.DrawOdds = Convert.ToDouble(betLineArray[counter], CultureInfo.GetCultureInfo("en-GB"));
                                    }
                                }
                                else if (charH.Last().Equals('A'))
                                {
                                    if (oddsValue.Equals(""))
                                    {
                                        odds.AwayOdds = 0.0;
                                    }
                                    else
                                    {
                                        odds.AwayOdds = Convert.ToDouble(betLineArray[counter], CultureInfo.GetCultureInfo("en-GB"));
                                    }
                                }
                                session.Update(odds);
                                tx.Commit();
                            }
                        }
                        counter++;
                    }
                    betLine = fileReader.ReadLine();
                }
            }
        }
    }
}