﻿using FootballPredictions.Mappings;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Web;

namespace FootballPredictions.DataHandling.PopulateDatabase
{
    /// <summary>
    /// Populates matches that have not been played yet. 
    /// These matches have the "Result" property set to 'N'.
    /// </summary>
    public class PopulateUpcomingMatches
    {
        //TODO: Test if this works
        public PopulateUpcomingMatches()
        {
            StreamReader fileReader = new StreamReader(Constants.TEXT_FILES + Constants.STAGE_ID + "/upcomingFixtures.csv");

            using (var session = NHibernateConfig.OpenSession())
            {
                var line = "";
                List<int> lineups;
                while ((line = fileReader.ReadLine()) != null)
                {
                    var lineArray = line.Split(',');
                    var matchId = Convert.ToInt32(lineArray[0]);
                    var homeTeamId = Convert.ToInt32(lineArray[1]);
                    var awayTeamId = Convert.ToInt32(lineArray[2]);
                    var homeGoals = 0;
                    var awayGoals = 0;
                    var date = DateTime.ParseExact(lineArray[5], "dd/MM/yy", CultureInfo.InvariantCulture);
                    lineups = new List<int>();
                    for (int i = 6; i < 28; i++)
                    {
                        lineups.Add(Convert.ToInt32(lineArray[i]));
                    }
                    using (var tx = session.BeginTransaction())
                    {
                        var match = new Match()
                        {
                            MatchId = matchId,
                            HomeTeam = session.Get<Team>(homeTeamId),
                            AwayTeam = session.Get<Team>(awayTeamId),
                            HomeGoals = homeGoals,
                            AwayGoals = awayGoals,
                            DateAndTime = date,
                            Result = 'N',
                            Stage = session.Get<Stage>(Constants.STAGE_ID),
                        };
                        session.SaveOrUpdate(match);
                        tx.Commit();

                        var homeLineup = new ProbableLineUp()
                        {
                            Player1 = session.Get<Player>(lineups[0]),
                            Player2 = session.Get<Player>(lineups[1]),
                            Player3 = session.Get<Player>(lineups[2]),
                            Player4 = session.Get<Player>(lineups[3]),
                            Player5 = session.Get<Player>(lineups[4]),
                            Player6 = session.Get<Player>(lineups[5]),
                            Player7 = session.Get<Player>(lineups[6]),
                            Player8 = session.Get<Player>(lineups[7]),
                            Player9 = session.Get<Player>(lineups[8]),
                            Player10 = session.Get<Player>(lineups[9]),
                            Player11 = session.Get<Player>(lineups[10]),
                            Team = session.Get<Team>(homeTeamId),
                            Match = session.Get<Match>(matchId)
                        };
                        session.SaveOrUpdate(homeLineup);
                        tx.Commit();

                        var awayLineup = new ProbableLineUp()
                        {
                            Player1 = session.Get<Player>(lineups[11]),
                            Player2 = session.Get<Player>(lineups[12]),
                            Player3 = session.Get<Player>(lineups[13]),
                            Player4 = session.Get<Player>(lineups[14]),
                            Player5 = session.Get<Player>(lineups[15]),
                            Player6 = session.Get<Player>(lineups[16]),
                            Player7 = session.Get<Player>(lineups[17]),
                            Player8 = session.Get<Player>(lineups[18]),
                            Player9 = session.Get<Player>(lineups[19]),
                            Player10 = session.Get<Player>(lineups[20]),
                            Player11 = session.Get<Player>(lineups[21]),
                            Team = session.Get<Team>(awayTeamId),
                            Match = session.Get<Match>(matchId)
                        };
                        session.SaveOrUpdate(awayLineup);
                        tx.Commit();
                    }
                }
            }
        }
    }
}