﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;

using FootballPredictions.Mappings;
using System.Globalization;

namespace FootballPredictions.Evaluation.Betting.BettingStrategies
{
    /// <summary>
    /// Implementation of the Fixed bet betting strategy.
    /// </summary>
    public class FixedBet : Betting
    {

        public FixedBet(int simulationNo, string bookmakerId, double safetyMargin = 0.0)
        {
            BettingStrategy = "FixedBet";
            Init(simulationNo, bookmakerId, safetyMargin);
        }

        public override void BetOnIndex(int i)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var prediction = Predictions[i];
                var match = session.Get<Match>(prediction.Match.MatchId);

                var oddsList = from o in session.Query<Odds>().ToList()
                               where o.Match.MatchId == match.MatchId
                               && o.Bookmaker.BookmakerId == Bookmaker.BookmakerId
                               select o;
                var odds = oddsList.First();

                var betsPlaced = new double[3];

                var bookmakersProbability = new double[] { 1.0 / odds.HomeOdds, 1.0 / odds.DrawOdds, 1.0 / odds.AwayOdds };

                var margin = 0.0;

                //Place bet on the outcome with the largest positive margin compared with bookmaker probability.
                if (prediction.HomeProbability > bookmakersProbability[0])
                {
                    margin = prediction.HomeProbability - bookmakersProbability[0]; 
                    betsPlaced[0] = Constants.UNIT;
                }
                if (prediction.DrawProbability > bookmakersProbability[1])
                {
                    if ((prediction.DrawProbability - bookmakersProbability[1]) > margin)
                    {
                        margin = prediction.DrawProbability - bookmakersProbability[1];
                        betsPlaced[1] = Constants.UNIT;
                        betsPlaced[0] = 0.0;
                    }
                }
                if (prediction.AwayProbability > bookmakersProbability[2])
                {
                    if ((prediction.AwayProbability - bookmakersProbability[2]) > margin)
                    {
                        margin = prediction.AwayProbability - bookmakersProbability[2];
                        betsPlaced[2] = Constants.UNIT;
                        betsPlaced[1] = 0.0;
                        betsPlaced[0] = 0.0;
                    }
                }

                CalculateReturn(match.Result, odds, prediction, betsPlaced);
            }
        }
    }
}