﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;

using FootballPredictions.Mappings;


namespace FootballPredictions.Evaluation.Betting.BettingStrategies
{
    /// <summary>
    /// The Fixed probability betting strategy.
    /// </summary>
    public class FixedProbabilityBet : Betting
    {

        public FixedProbabilityBet(int simulationNo, string bookmakerId, double safetyMargin)
        {
            BettingStrategy = "FixedProbabilityBet";
            Init(simulationNo, bookmakerId, safetyMargin);
        }

        public override void BetOnIndex(int i)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var prediction = Predictions[i];
                var match = session.Get<Match>(prediction.Match.MatchId);

                var oddsList = from o in session.Query<Odds>().ToList()
                               where o.Match.MatchId == match.MatchId
                               && o.Bookmaker.BookmakerId == Bookmaker.BookmakerId
                               select o;
                var odds = oddsList.First();

                var betsPlaced = new double[3];

                var bookmakersProbability = new double[] { (1/odds.HomeOdds) / ((1/odds.HomeOdds) + (1/odds.DrawOdds) + (1/odds.AwayOdds)),
                                                            (1/odds.DrawOdds) / ((1/odds.HomeOdds) + (1/odds.DrawOdds) + (1/odds.AwayOdds)),
                                                            (1/odds.AwayOdds) / ((1/odds.HomeOdds) + (1/odds.DrawOdds) + (1/odds.AwayOdds))};

                if (bookmakersProbability[0] < (prediction.HomeProbability - SafetyMargin) && prediction.HomeProbability > 0.487)
                {
                    betsPlaced[0] = Constants.UNIT;
                }
                else if (bookmakersProbability[1] < (prediction.DrawProbability - SafetyMargin * 1.25) && prediction.DrawProbability > 0.267)
                {
                    betsPlaced[1] = Constants.UNIT;
                }
                else if (bookmakersProbability[2] < (prediction.AwayProbability - SafetyMargin * 1.5) && prediction.AwayProbability > 0.246)
                {
                    betsPlaced[2] = Constants.UNIT;
                }
                CalculateReturn(match.Result, odds, prediction, betsPlaced);
            }
        }
    }
}