﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;

using FootballPredictions.Mappings;
using System.Globalization;

namespace FootballPredictions.Evaluation.Betting.BettingStrategies
{
    /// <summary>
    /// The Fixed Return betting strategy.
    /// </summary>
    public class FixedReturn : Betting
    {
        public FixedReturn(int simulationNo, string bookmakerId, double safetyMargin = 0.0)
        {
            BettingStrategy = "FixedReturn";
            Init(simulationNo, bookmakerId, safetyMargin);
        }

        public override void BetOnIndex(int i)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var prediction = Predictions[i];
                var match = session.Get<Match>(prediction.Match.MatchId);

                var oddsList = from o in session.Query<Odds>().ToList()
                               where o.Match.MatchId == match.MatchId
                               && o.Bookmaker.BookmakerId == Bookmaker.BookmakerId
                               select o;
                var odds = oddsList.First();

                var betsPlaced = new double[3];

                var bookmakersProbability = new double[] { 1.0 / odds.HomeOdds, 1.0 / odds.DrawOdds, 1.0 / odds.AwayOdds };

                if (prediction.HomeProbability > bookmakersProbability[0] + SafetyMargin)
                {
                    betsPlaced[0] = Constants.UNIT / odds.HomeOdds;
                }
                if (prediction.DrawProbability > bookmakersProbability[1] + SafetyMargin)
                {
                    betsPlaced[1] = Constants.UNIT / odds.DrawOdds;
                }
                if (prediction.AwayProbability > bookmakersProbability[2] + SafetyMargin)
                {
                    betsPlaced[2] = Constants.UNIT / odds.AwayOdds;
                }

                CalculateReturn(match.Result, odds, prediction, betsPlaced);
            }
        }
    }
}