﻿using FootballPredictions.Mappings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;

namespace FootballPredictions.Evaluation.Betting.BettingStrategies
{
    /// <summary>
    /// The Kelly betting strategy. This should be implemented again, with a fixed bankroll, to work as intended.
    /// </summary>
    public class Kelly : Betting
    {
        
        public Kelly(int simulationNo, string bookmakerId, double safetyMargin)
        {
            BettingStrategy = "Kelly";
            Init(simulationNo, bookmakerId, safetyMargin);
        }

        public override void BetOnIndex(int i)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var prediction = Predictions[i];
                var match = session.Get<Match>(prediction.Match.MatchId);

                var oddsList = from o in session.Query<Odds>().ToList()
                               where o.Match.MatchId == match.MatchId
                               && o.Bookmaker.BookmakerId == Bookmaker.BookmakerId
                               select o;
                var odds = oddsList.First();

                var betsPlaced = new double[3];

                betsPlaced[0] = Constants.UNIT
                                * (prediction.HomeProbability * (odds.HomeOdds*Constants.UNIT - Constants.UNIT + 1) - 1)
                                / (odds.HomeOdds*Constants.UNIT - Constants.UNIT);
                betsPlaced[1] = Constants.UNIT
                                * (prediction.DrawProbability * (odds.DrawOdds*Constants.UNIT - Constants.UNIT + 1) - 1)
                                / (odds.DrawOdds*Constants.UNIT - Constants.UNIT);
                betsPlaced[2] = Constants.UNIT
                                * (prediction.AwayProbability * (odds.AwayOdds*Constants.UNIT - Constants.UNIT + 1) - 1)
                                / (odds.AwayOdds*Constants.UNIT - Constants.UNIT);
                betsPlaced[0] = betsPlaced[0] > Math.Max(betsPlaced[1], betsPlaced[2]) ? betsPlaced[0] : 0;
                betsPlaced[1] = betsPlaced[1] > Math.Max(betsPlaced[0], betsPlaced[2]) ? betsPlaced[1] : 0;
                betsPlaced[2] = betsPlaced[2] > Math.Max(betsPlaced[0], betsPlaced[1]) ? betsPlaced[2] : 0;

                CalculateReturn(match.Result, odds, prediction, betsPlaced);
            }
        }
    }
}