﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;

using FootballPredictions.Mappings;

namespace FootballPredictions.Evaluation.Betting.BettingStrategies
{
    /// <summary>
    /// This strategy looks at the predicted probabilities, without cosidering the bookmaker's odds.
    /// A bet is placed on draw if the expected probability is above 26.7 %, away if it is above 24.6 %
    /// and home if it is above 48.7 % (as found by Clarke and Norman (1995)).
    /// </summary>
    public class ProbabilityBet : Betting
    {
        public ProbabilityBet(int simulationNo, string bookmakerid, double safetyMargin = 0.0)
        {
            BettingStrategy = "ProbabilityBet";
            Init(simulationNo, bookmakerid, safetyMargin);
        }
        
        public override void BetOnIndex(int i)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var prediction = Predictions[i];
                var match = session.Get<Match>(prediction.Match.MatchId);

                var oddsList = from o in session.Query<Odds>().ToList()
                               where o.Match.MatchId == match.MatchId
                               && o.Bookmaker.BookmakerId == Bookmaker.BookmakerId
                               select o;
                var odds = oddsList.First();

                var betsPlaced = new double[3];

                if (prediction.DrawProbability > 0.267)
                {
                    betsPlaced[1] = Constants.UNIT;
                }
                else if (prediction.AwayProbability > 0.246)
                {
                    betsPlaced[2] = Constants.UNIT;
                }
                else
                {
                    betsPlaced[0] = Constants.UNIT;
                }

                CalculateReturn(match.Result, odds, prediction, betsPlaced);
            }
        }
    }
}