﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;

using FootballPredictions.Mappings;

namespace FootballPredictions.Evaluation.Betting.BettingStrategies
{
    /// <summary>
    /// Betting strategy adapted from wallpaperingfog.co.uk.
    /// </summary>
    public class Wallpapering : Betting
    {
        public Wallpapering(int simulationNo, string bookmakerId, double safetyMargin)
        {
            BettingStrategy = "Wallpapering";
            Init(simulationNo, bookmakerId, safetyMargin);
        }

        public override void BetOnIndex(int i)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var prediction = Predictions[i];
                var match = session.Get<Match>(prediction.Match.MatchId);

                var oddsList = from o in session.Query<Odds>().ToList()
                               where o.Match.MatchId == match.MatchId
                               && o.Bookmaker.BookmakerId == Bookmaker.BookmakerId
                               select o;
                var odds = oddsList.First();

                var betsPlaced = new double[3];

                var bookmakersProbability = new double[] { (1/odds.HomeOdds) / ((1/odds.HomeOdds) + (1/odds.DrawOdds) + (1/odds.AwayOdds)),
                                                            (1/odds.DrawOdds) / ((1/odds.HomeOdds) + (1/odds.DrawOdds) + (1/odds.AwayOdds)),
                                                            (1/odds.AwayOdds) / ((1/odds.HomeOdds) + (1/odds.DrawOdds) + (1/odds.AwayOdds))};

                if(prediction.DrawProbability > 0.27)
                {
                    betsPlaced[1] = Constants.UNIT;
                }
                else if(prediction.HomeProbability > Math.Max(prediction.DrawProbability, prediction.AwayProbability))
                {
                    betsPlaced[0] = Constants.UNIT;
                }
                else if(prediction.AwayProbability > Math.Max(prediction.HomeProbability, prediction.DrawProbability))
                {
                    betsPlaced[2] = Constants.UNIT;
                }
                
                CalculateReturn(match.Result, odds, prediction, betsPlaced);
            }
        }
    }
}