﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;

namespace FootballPredictions.Evaluation.Betting.Plots
{
    /// <summary>
    /// Plots the accumulated income.
    /// </summary>
    public class AccumulatedIncome : BettingPlot
    {

        public AccumulatedIncome()
        {
        }

        /// <summary>
        /// Plot the accumulated income to a .tex file.
        /// </summary>
        public override void Plot()
        {
            using (System.IO.StreamWriter file = new System.IO.StreamWriter(
                Constants.TEXT_FILES + "JAGS/Betting/"
                + SimulationName
                + "_"
                + Stage.ToString()
                + "_"
                + BettingStrategy
                + "_accumulatedIncome.tex"))
            {

                var prefix = "\\begin{figure} \n"
                            + "\\centering \n"
                            + "\\begin{tikzpicture} \n"
                            + "\\begin{axis}\n"
                            + "[legend pos=outer north east, \n"
                            + "xlabel=Match, ylabel=Accumulated Income]";

                file.WriteLine(prefix);

                file.WriteLine("\\addplot[color=red,mark=x] coordinates {");
                var counter = 0;
                var incomeSoFar = 0.0;
                var betsPlacedSoFar = 0.0;
                file.WriteLine("(" + counter + "," + incomeSoFar.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")");
                foreach (var b in Bets)
                {
                    counter++;
                    incomeSoFar += b.Return;
                    betsPlacedSoFar += b.BetsPlaced[0] + b.BetsPlaced[1] + b.BetsPlaced[2];
                    file.WriteLine("(" + counter + "," + incomeSoFar.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")");
                }
                file.WriteLine("};");
                var suffix = "\\legend{" + SimulationName + ": Accumulated Income} \n"
                            + "\\end{axis} \n"
                            + "\\end{tikzpicture}\\newline"
                            + "\\caption{Total bets placed: +" + betsPlacedSoFar + "\nTotal winnings: " + incomeSoFar + "\nProfit: " + (incomeSoFar/betsPlacedSoFar) + "}% \n"
                            + "\\label{pgfplots:<ENTER Caption>}% \n"
                            + "\\end{figure}";
                file.WriteLine(suffix);
                file.Write(Constants.StringOfConstants());
            }
        }
    }
}