﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Web;

namespace FootballPredictions.Evaluation.Betting.Plots
{
    /// <summary>
    /// A plot with dots showing the bets as a green or red mark if the bet was successfull or not.
    /// </summary>
    public class BetProbabilityPlot : BettingPlot
    {
        public override void Plot()
        {
            var prefix = "\\begin{figure}\n"
            + "\\centering\n"
            + "\\begin{tikzpicture}\n"
            + "\\begin{axis} [xmin=0,xmax=1, xlabel=Probability, ylabel=Return]\n"
            + "\\addplot[scatter,only marks,\n"
            + "point meta=explicit symbolic,\n"
            + "mark size=0.6pt,\n"
            + "scatter/classes=\n"
            + "{a={mark=o,green},b={mark=o,red},c={mark=o,black}}]\n"
            + "table[meta=label] {\n"
            + "x y label\n";

            var coordinates = "";
            foreach (var b in Bets)
            {
                var x = 0.0;
                var y = 0.0;

                if (b.BetsPlaced[0] > 0)
                {
                    x = b.Prediction.HomeProbability;
                    y = b.Return;
                }
                else if (b.BetsPlaced[1] > 0)
                {
                    x = b.Prediction.DrawProbability;
                    y = b.Return;
                }
                else if (b.BetsPlaced[2] > 0)
                {
                    x = b.Prediction.AwayProbability;
                    y = b.Return;
                }
                
                var label = b.Return > 0 ? "a" : "b";
                if (b.Return == 0)
                { 
                    label = "c"; 
                }

                coordinates += x.ToString(CultureInfo.GetCultureInfo("en-GB")) + " " + y.ToString(CultureInfo.GetCultureInfo("en-GB")) + " " + label + "\n";
            }
            var suffix = "};\n"
                       + "\\addplot [black, sharp plot, update limits=false] coordinates {\n"
                       + "(0,0)(1,0)};\n"
                       + "\\end{axis}\n"
                       + "\\end{tikzpicture}\n"
                       + "\\end{figure}\n";
            using(StreamWriter file = new StreamWriter(Constants.TEXT_FILES 
                + "JAGS/Betting/" + SimulationName + "_" + Stage + "_" + BettingStrategy + "_BetReturnPlot.tex"))
            {
                file.Write(prefix+coordinates+suffix);
            }
        }
    }
}