﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Web;

namespace FootballPredictions.Evaluation.Betting.Plots
{
    /// <summary>
    /// Plot with marks if the bets were successfull or not.
    /// </summary>
    public class BetsPlacedOverview : BettingPlot
    {
        public override void Plot()
        {
            var prefix = "\\begin{figure}\n"
                       + "\\centering\n"
                       + "\\begin{tikzpicture}\n"
                       + "\\begin{axis} [xmin=0,xmax=1,ymin=0,ymax=20,xlabel=Probability, ylabel=Odds]\n"
                       + "\\addplot[domain=0:1, samples=200] {1/x};\n"
                       + "\\addplot[scatter,only marks,\n"
                       + "point meta=explicit symbolic,\n"
                       + "mark size=0.6pt,\n"
                       + "scatter/classes=\n"
                       + "{a={mark=o,green},b={mark=o,red},c={mark=o,black}}]\n"
                       + "table[meta=label] {\n"
                       + "x y label\n";

            var coordinates = "";
            foreach (var b in Bets)
            {
                var x = 0.0;
                var y = 0.0;

                if (b.BetsPlaced[0] > 0)
                {
                    x = b.Prediction.HomeProbability;
                    y = b.Odds.HomeOdds;
                }
                else if (b.BetsPlaced[1] > 0)
                {
                    x = b.Prediction.DrawProbability;
                    y = b.Odds.DrawOdds;
                }
                else if (b.BetsPlaced[2] > 0)
                {
                    x = b.Prediction.AwayProbability;
                    y = b.Odds.AwayOdds;
                }
                
                var label = b.Return > 0 ? "a" : "b";
                if (b.Return == 0)
                { 
                    label = "c"; 
                }

                coordinates += x.ToString(CultureInfo.GetCultureInfo("en-GB")) + " " + y.ToString(CultureInfo.GetCultureInfo("en-GB")) + " " + label + "\n";
            }
            var suffix = "};\n"
                       + "\\end{axis}\n"
                       + "\\end{tikzpicture}\n"
                       + "\\end{figure}\n";
            using(StreamWriter file = new StreamWriter(Constants.TEXT_FILES
                + "JAGS/Betting/" + SimulationName + "_" + Stage + "_" + BettingStrategy + "_BetsPlacedOverview.tex"))
            {
                file.Write(prefix+coordinates+suffix);
            }
        }
    }
}