﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;

namespace FootballPredictions.Evaluation.Betting.Plots
{
    public class ExpectedReturn : BettingPlot
    {
        public override void Plot()
        {
            var prefix =  "\\begin{figure} \n"
                        + "\\centering \n"
                        + "\\begin{tikzpicture} \n"
                        + "\\begin{axis}\n"
                        + "[legend pos=outer north east, \n"
                        + "xlabel=Match, ylabel=Expected return] \n";

            var suffix =  "\\legend{Expected return} \n"
                        + "\\end{axis} \n"
                        + "\\end{tikzpicture}\\newline"
                        + "\\caption{<ENTER Figure Text>}% \n"
                        + "\\label{pgfplots:<ENTER Caption>}% \n"
                        + "\\end{figure}\n";

            var homePlot = prefix;
            homePlot += CreateLinePlot('H');
            homePlot += suffix;

            var drawPlot = prefix;
            drawPlot += CreateLinePlot('D');
            drawPlot += suffix;

            var awayPlot = prefix;
            awayPlot += CreateLinePlot('A');
            awayPlot += suffix;

            using (System.IO.StreamWriter file = new System.IO.StreamWriter(Constants.TEXT_FILES + "/JAGS/BETTING/expectedReturn_" + SimulationName + "_" + Stage + "_" + BettingStrategy + ".tex"))
            {
                file.Write(homePlot + drawPlot + awayPlot);
            }
        }

        private string CreateLinePlot(char version)
        {
            var plot = "";

            plot += "\\addplot[] coordinates {\n";
            for (int j = 0; j < Bets.Count(); j++)
            {

                switch (version)
                {
                    case ('H'):
                        {
                            plot += "(" + j + "," + (Bets[j].Prediction.HomeProbability * Bets[j].Odds.HomeOdds).ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            break;
                        }
                    case ('D'):
                        {
                            plot += "(" + j + "," + (Bets[j].Prediction.DrawProbability * Bets[j].Odds.DrawOdds).ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            break;
                        }
                    case ('A'):
                        {
                            plot += "(" + j + "," + (Bets[j].Prediction.AwayProbability * Bets[j].Odds.AwayOdds).ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            break;
                        }
                    default: break;
                }
            }
            plot += "};\n";

            return plot;
        }
    }
}