﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;

namespace FootballPredictions.Evaluation.Betting.Plots
{
    /// <summary>
    /// Write a table with all the statistics to file.
    /// </summary>
    public class StatisticsTable : BettingPlot
    {
        public DateTime StartDate { get; set; }
        private double[] AvgFavoriteOdds = new double[2];
        private double[] AvgFavoriteProbability = new double[2];
        private double[] AvgHomeOdds = new double[2];
        private double[] AvgDrawOdds = new double[2];
        private double[] AvgAwayOdds = new double[2];
        private double[] AvgBookieHomeProbability = new double[2];
        private double[] AvgBookieDrawProbability = new double[2];
        private double[] AvgBookieAwayProbability = new double[2];
        private double[] AvgHomeProbability = new double[2];
        private double[] AvgDrawProbability = new double[2];
        private double[] AvgAwayProbability = new double[2];
        private double[] AvgDistanceFromOdds = new double[2];
        private double[] AvgDistanceFromHomeOdds = new double[2];
        private double[] AvgDistanceFromDrawOdds = new double[2];
        private double[] AvgDistanceFromAwayOdds = new double[2];
        private double[] AvgOddsOnBets = new double[2];
        private double[] AvgOddsOnHomeBets = new double[2];
        private double[] AvgOddsOnDrawBets = new double[2];
        private double[] AvgOddsOnAwayBets = new double[2];
        private int[] NumberOfMatches = new int[2];
        private int[] NumberOfHomeVictories = new int[2];
        private int[] NumberOfDrawMatches = new int[2];
        private int[] NumberOfAwayVictories = new int[2];
        private int[] NumberOfHomeBets = new int[2];
        private int[] NumberOfDrawBets = new int[2];
        private int[] NumberOfAwayBets = new int[2];
        private int[] NumberOfBetsNegExpReturn = new int[2];
        private int[] NumberOfBetsOddsGTFive = new int[2];
        private int[] NumberOfWonHomeBets = new int[2];
        private int[] NumberOfWonDrawBets = new int[2];
        private int[] NumberOfWonAwayBets = new int[2];
        private int[] NumberOfWonBetsNegExpReturn = new int[2];
        private int[] NumberOfWonBetsOddsGTFive = new int[2];
        private double[] AmountBetHome = new double[2];
        private double[] AmountBetDraw = new double[2];
        private double[] AmountBetAway = new double[2];
        private double[] AvgAmountBetHome = new double[2];
        private double[] AvgAmountBetDraw = new double[2];
        private double[] AvgAmountBetAway = new double[2];
        private double[] AvgReturnHomeBets = new double[2];
        private double[] AvgReturnDrawBets = new double[2];
        private double[] AvgReturnAwayBets = new double[2];
        private double[] ReturnWonHomeBets = new double[2];
        private double[] ReturnWonDrawBets = new double[2];
        private double[] ReturnWonAwayBets = new double[2];
        private double[] ReturnFromHomeBets = new double[2];
        private double[] ReturnFromDrawBets = new double[2];
        private double[] ReturnFromAwayBets = new double[2];
        private double[] ReturnFromBetsNegExpRet = new double[2];
        private double[] ReturnFromBetsOddsGTFive = new double[2];

        public override void Plot()
        {
            var prefix = "\\begin{table}% \n"
                         + "\\centering \n"
                         + "\\caption{Table of statistics for stage " + Stage + ", prediction model " + SimulationName + ", betting strategy " + BettingStrategy + "} \n"
                         + "\\label{} \n"
                         + "\\begin{tabular}{ | c || c || c |} \n"
                         + "\\hline \n"
                         + "Variable & Before "+StartDate.Day+"."+StartDate.Month+"."+StartDate.Year+" & After "+StartDate.Day+"."+StartDate.Month+"."+StartDate.Year+" \\\\ \n";

            var suffix = "\\hline \n"
                        + "\\end{tabular} \n"
                        + "\\end{table} \n";

            var tableValues = CalculateStatistics();

            using (var file = new StreamWriter(Constants.TEXT_FILES + "JAGS/Betting/statistics_" + Stage + "_" + SimulationName + "_" + BettingStrategy + ".tex"))
            {
                file.Write(prefix + tableValues + suffix);
            }
        }

        private string CalculateStatistics()
        {
            var afterStartDate = 0;
            var tableValues = "";
            foreach (var b in Bets)
            {
                if (b.Match.DateAndTime.CompareTo(StartDate) == 1) afterStartDate = 1;
                
                // General odds stats
                NumberOfMatches[afterStartDate]++;
                AvgHomeOdds[afterStartDate]              += b.Odds.HomeOdds;
                AvgBookieHomeProbability[afterStartDate] += 1.0 / b.Odds.HomeOdds;
                AvgHomeProbability[afterStartDate]       += b.Prediction.HomeProbability;
                AvgDrawOdds[afterStartDate]              += b.Odds.DrawOdds;
                AvgBookieDrawProbability[afterStartDate] += 1.0 / b.Odds.DrawOdds;
                AvgDrawProbability[afterStartDate]       += b.Prediction.DrawProbability;
                AvgAwayOdds[afterStartDate]              += b.Odds.AwayOdds;
                AvgBookieAwayProbability[afterStartDate] += 1.0 / b.Odds.AwayOdds;
                AvgAwayProbability[afterStartDate]       += b.Prediction.AwayProbability;
                AvgFavoriteOdds[afterStartDate]          += new double[] { b.Odds.HomeOdds, b.Odds.DrawOdds, b.Odds.AwayOdds }.Min();
                AvgFavoriteProbability[afterStartDate]   += new double[] { b.Prediction.HomeProbability, b.Prediction.DrawProbability, b.Prediction.AwayProbability }.Max();
                AvgDistanceFromHomeOdds[afterStartDate]  += Math.Abs((1.0/b.Odds.HomeOdds) - b.Prediction.HomeProbability);
                AvgDistanceFromDrawOdds[afterStartDate]  += Math.Abs((1.0/b.Odds.DrawOdds) - b.Prediction.DrawProbability);
                AvgDistanceFromAwayOdds[afterStartDate]  += Math.Abs((1.0/b.Odds.AwayOdds) - b.Prediction.AwayProbability);

                //Count outcomes
                switch (b.Match.Result)
                {
                    case ('H'):
                        {
                            NumberOfHomeVictories[afterStartDate]++;
                            break;
                        }
                    case ('D'):
                        {
                            NumberOfDrawMatches[afterStartDate]++;
                            break;
                        }
                    case ('A'):
                        {
                            NumberOfAwayVictories[afterStartDate]++;
                            break;
                        }
                    default: break;
                }

                // Betting stats
                if (b.BetsPlaced[0] > 0)
                {
                    NumberOfHomeBets[afterStartDate]++;
                    AvgOddsOnBets[afterStartDate] += b.Odds.HomeOdds;
                    AvgOddsOnHomeBets[afterStartDate] += b.Odds.HomeOdds;

                    if (b.Match.Result == 'H')
                    {
                        NumberOfWonHomeBets[afterStartDate]++;
                        ReturnWonHomeBets[afterStartDate] += b.Return;
                    }
                    AmountBetHome[afterStartDate] += b.BetsPlaced[0];
                    ReturnFromHomeBets[afterStartDate] += b.Return;
                    if (b.Odds.HomeOdds * b.Prediction.HomeProbability < 1)
                    {
                        NumberOfBetsNegExpReturn[afterStartDate]++;
                        ReturnFromBetsNegExpRet[afterStartDate] += b.Return;
                        if (b.Match.Result == 'H')
                        {
                            NumberOfWonBetsNegExpReturn[afterStartDate]++;
                        }
                    }
                    if (b.Odds.HomeOdds > 5)
                    {
                        NumberOfBetsOddsGTFive[afterStartDate]++;
                        ReturnFromBetsOddsGTFive[afterStartDate] += b.Return;
                        if (b.Match.Result == 'H')
                        {
                            NumberOfWonBetsOddsGTFive[afterStartDate]++;
                        }
                    }
                }
                 
                if (b.BetsPlaced[1] > 0)
                {
                    NumberOfDrawBets[afterStartDate]++;
                    AvgOddsOnBets[afterStartDate] += b.Odds.DrawOdds;
                    AvgOddsOnDrawBets[afterStartDate] += b.Odds.DrawOdds;
                    if (b.Match.Result == 'D')
                    {
                        NumberOfWonDrawBets[afterStartDate]++;
                        ReturnWonDrawBets[afterStartDate] += b.Return;
                    }
                    AmountBetDraw[afterStartDate] += b.BetsPlaced[1];
                    ReturnFromDrawBets[afterStartDate] += b.Return;
                    if (b.Odds.DrawOdds * b.Prediction.DrawProbability < 1)
                    {
                        NumberOfBetsNegExpReturn[afterStartDate]++;
                        ReturnFromBetsNegExpRet[afterStartDate] += b.Return;
                        if (b.Match.Result == 'D')
                        {
                            NumberOfWonBetsNegExpReturn[afterStartDate]++;
                        }
                    }
                    if (b.Odds.DrawOdds > 5)
                    {
                        NumberOfBetsOddsGTFive[afterStartDate]++;
                        ReturnFromBetsOddsGTFive[afterStartDate] += b.Return;
                        if (b.Match.Result == 'D')
                        {
                            NumberOfWonBetsOddsGTFive[afterStartDate]++;
                        }
                    }
                }
                if (b.BetsPlaced[2] > 0)
                {
                    NumberOfAwayBets[afterStartDate]++;
                    AvgOddsOnBets[afterStartDate] += b.Odds.AwayOdds;
                    AvgOddsOnAwayBets[afterStartDate] += b.Odds.AwayOdds;
                    if (b.Match.Result == 'A')
                    {
                        NumberOfWonAwayBets[afterStartDate]++;
                        ReturnWonAwayBets[afterStartDate] += b.Return;
                    }
                    AmountBetAway[afterStartDate] += b.BetsPlaced[2];
                    ReturnFromAwayBets[afterStartDate] += b.Return;
                    if (b.Odds.AwayOdds * b.Prediction.AwayProbability < 1)
                    {
                        NumberOfBetsNegExpReturn[afterStartDate]++;
                        ReturnFromBetsNegExpRet[afterStartDate] += b.Return;
                        if (b.Match.Result == 'A')
                        {
                            NumberOfWonBetsNegExpReturn[afterStartDate]++;
                        }
                    }
                    if (b.Odds.AwayOdds > 5)
                    {
                        NumberOfBetsOddsGTFive[afterStartDate]++;
                        ReturnFromBetsOddsGTFive[afterStartDate] += b.Return;
                        if (b.Match.Result == 'A')
                        {
                            NumberOfWonBetsOddsGTFive[afterStartDate]++;
                        }
                    }
                }
            }

            // Calculate averages
            for (int i = 0; i < 2; i++)
            {
                AvgFavoriteOdds[i] = AvgFavoriteOdds[i] / (double)NumberOfMatches[i];
                AvgFavoriteProbability[i] = AvgFavoriteProbability[i] / (double)NumberOfMatches[i];
                AvgAmountBetHome[i] = AvgAmountBetHome[i] / (double)NumberOfHomeBets[i];
                AvgAmountBetDraw[i] = AvgAmountBetDraw[i] / (double)NumberOfDrawBets[i];
                AvgAmountBetAway[i] = AvgAmountBetAway[i] / (double)NumberOfAwayBets[i];
                AvgHomeOdds[i] = AvgHomeOdds[i] / (double)NumberOfMatches[i];
                AvgDrawOdds[i] = AvgDrawOdds[i] / (double)NumberOfMatches[i];
                AvgAwayOdds[i] = AvgAwayOdds[i] / (double)NumberOfMatches[i];
                AvgBookieHomeProbability[i] = AvgBookieHomeProbability[i] / (double)NumberOfMatches[i];
                AvgBookieDrawProbability[i] = AvgBookieDrawProbability[i] / (double)NumberOfMatches[i];
                AvgBookieAwayProbability[i] = AvgBookieAwayProbability[i] / (double)NumberOfMatches[i];
                AvgHomeProbability[i] = AvgHomeProbability[i] / (double)NumberOfMatches[i];
                AvgDrawProbability[i] = AvgDrawProbability[i] / (double)NumberOfMatches[i];
                AvgAwayProbability[i] = AvgAwayProbability[i] / (double)NumberOfMatches[i];
                AvgReturnHomeBets[i] = ReturnFromHomeBets[i] / (double)NumberOfHomeBets[i];
                AvgReturnDrawBets[i] = ReturnFromDrawBets[i] / (double)NumberOfDrawBets[i];
                AvgReturnAwayBets[i] = ReturnFromAwayBets[i] / (double)NumberOfAwayBets[i];
                AvgDistanceFromHomeOdds[i] = AvgDistanceFromHomeOdds[i] / (double)NumberOfMatches[i];
                AvgDistanceFromDrawOdds[i] = AvgDistanceFromDrawOdds[i] / (double)NumberOfMatches[i];
                AvgDistanceFromAwayOdds[i] = AvgDistanceFromAwayOdds[i] / (double)NumberOfMatches[i];
                AvgOddsOnBets[i] = AvgOddsOnBets[i] / ((double)NumberOfHomeBets[i] + (double)NumberOfDrawBets[i] + (double)NumberOfAwayBets[i]);
                AvgOddsOnHomeBets[i] = AvgOddsOnHomeBets[i] / (double)NumberOfHomeBets[i];
                AvgOddsOnDrawBets[i] = AvgOddsOnDrawBets[i] / (double)NumberOfDrawBets[i];
                AvgOddsOnAwayBets[i] = AvgOddsOnAwayBets[i] / (double)NumberOfAwayBets[i];
            }
            AvgDistanceFromOdds[0] = (AvgDistanceFromHomeOdds[0] + AvgDistanceFromDrawOdds[0] + AvgDistanceFromAwayOdds[0]) / 3.0;
            AvgDistanceFromOdds[1] = (AvgDistanceFromHomeOdds[1] + AvgDistanceFromDrawOdds[1] + AvgDistanceFromAwayOdds[1]) / 3.0;

            tableValues += "Number of matches & "               + NumberOfMatches[0] + " & "                            + NumberOfMatches[1]        + "\\\\ \n";
            tableValues += "Number of home victories & "        + NumberOfHomeVictories[0] + " & "                      + NumberOfHomeVictories[1]  + "\\\\ \n";
            tableValues += "Number of draws & "                 + NumberOfDrawMatches[0] + " & "                        + NumberOfDrawMatches[1]    + "\\\\ \n";
            tableValues += "Number of away victories & "        + NumberOfAwayVictories[0] + " & "                      + NumberOfAwayVictories[1] + "\\\\ \n";          
            tableValues += "Avg favorite odds & "               + AvgFavoriteOdds[0].ToString("F3") +" & "              + AvgFavoriteOdds[1].ToString("F3")  + "\\\\ \n";               
            tableValues += "Avg home odds & "                   + AvgHomeOdds[0].ToString("F3") + " & "                 + AvgHomeOdds[1].ToString("F3")  + "\\\\ \n";
            tableValues += "Avg draw odds & "                   + AvgDrawOdds[0].ToString("F3") + " & "                 + AvgDrawOdds[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg away odds & "                   + AvgAwayOdds[0].ToString("F3") + " & "                 + AvgAwayOdds[1].ToString("F3") + "\\\\ \n";                   
            tableValues += "Avg favorite odds probability & "   + (1.0/AvgFavoriteOdds[0]).ToString("F3") + " & "       + (1.0 / AvgFavoriteOdds[1]).ToString("F3") + "\\\\ \n";   
            tableValues += "Avg favorite probability & "        + AvgFavoriteProbability[0].ToString("F3") + " & "      + AvgFavoriteProbability[1].ToString("F3") + "\\\\ \n";    
            tableValues += "Avg 1 / Home odds & "               + AvgBookieHomeProbability[0].ToString("F3") + " & "    + AvgBookieHomeProbability[1].ToString("F3") + "\\\\ \n";  
            tableValues += "Avg home probability & "            + AvgHomeProbability[0].ToString("F3") + " & "          + AvgHomeProbability[1].ToString("F3") + "\\\\ \n";        
            tableValues += "Avg 1 / Draw odds & "               + AvgBookieDrawProbability[0].ToString("F3") + " & "    + AvgBookieDrawProbability[1].ToString("F3") + "\\\\ \n";  
            tableValues += "Avg draw probability & "            + AvgDrawProbability[0].ToString("F3") + " & "          + AvgDrawProbability[1].ToString("F3") + "\\\\ \n";        
            tableValues += "Avg 1 / Away odds & "               + AvgBookieAwayProbability[0].ToString("F3") + " & "    + AvgBookieAwayProbability[1].ToString("F3") + "\\\\ \n";  
            tableValues += "Avg away probability & "            + AvgAwayProbability[0].ToString("F3") + " & "          + AvgAwayProbability[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg distance from odds & "          + AvgDistanceFromOdds[0].ToString("F3") + " & "         + AvgDistanceFromOdds[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg distance from home odds & "     + AvgDistanceFromHomeOdds[0].ToString("F3") + " & "     + AvgDistanceFromHomeOdds[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg distance from draw odds & "     + AvgDistanceFromDrawOdds[0].ToString("F3") + " & "     + AvgDistanceFromDrawOdds[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg distance from away odds & "     + AvgDistanceFromAwayOdds[0].ToString("F3") + " & "     + AvgDistanceFromAwayOdds[1].ToString("F3") + "\\\\ \n";
            tableValues += "Total number of bets & "            + (NumberOfHomeBets[0] + NumberOfDrawBets[0] + NumberOfAwayBets[0]) + " & "         
                + (NumberOfHomeBets[1] + NumberOfDrawBets[1] + NumberOfAwayBets[1]) + "\\\\ \n";
            tableValues += "Total amount bet & "                + (AmountBetHome[0] + AmountBetDraw[0] + AmountBetAway[0]) + " & " 
                + (AmountBetHome[1] + AmountBetDraw[1] + AmountBetAway[1]) + "\\\\ \n";
            tableValues += "Total profit & "                    + (ReturnFromHomeBets[0] + ReturnFromDrawBets[0] + ReturnFromAwayBets[0]) + " & "
                + (ReturnFromHomeBets[1] + ReturnFromDrawBets[1] + ReturnFromAwayBets[1]) + "\\\\ \n";
            tableValues += "Total profit \\% & " +              (100.0 * (ReturnFromHomeBets[0] + ReturnFromDrawBets[0] + ReturnFromAwayBets[0]) / (AmountBetHome[0] + AmountBetDraw[0] + AmountBetAway[0])).ToString("F3") + "\\% & " 
                + (100.0 * (ReturnFromHomeBets[1] + ReturnFromDrawBets[1] + ReturnFromAwayBets[1]) / (AmountBetHome[1] + AmountBetDraw[1] + AmountBetAway[1])).ToString("F3") + "\\% \\\\ \n";
            tableValues += "Number of home bets & "             + NumberOfHomeBets[0] + " & "                           + NumberOfHomeBets[1] + "\\\\ \n";
            tableValues += "Number of draw bets & "             + NumberOfDrawBets[0] + " & "                           + NumberOfDrawBets[1] + "\\\\ \n";
            tableValues += "Number of away bets & "             + NumberOfAwayBets[0] + " & "                           + NumberOfAwayBets[1] + "\\\\ \n";
            tableValues += "Number of won home bets & "         + NumberOfWonHomeBets[0] + " & "                        + NumberOfWonHomeBets[1] + "\\\\ \n";
            tableValues += "Number of won draw bets & "         + NumberOfWonDrawBets[0] + " & "                        + NumberOfWonDrawBets[1] + "\\\\ \n";
            tableValues += "Number of won away bets & "         + NumberOfWonAwayBets[0] + " & "                        + NumberOfWonAwayBets[1] + "\\\\ \n";
            tableValues += "Avg return home bets & "            + AvgReturnHomeBets[0].ToString("F3") + " & "           + AvgReturnHomeBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg return draw bets & "            + AvgReturnDrawBets[0].ToString("F3") + " & "           + AvgReturnDrawBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg return away bets & "            + AvgReturnAwayBets[0].ToString("F3") + " & "           + AvgReturnAwayBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Money earned home bets & "          + ReturnFromHomeBets[0].ToString("F3") + " & "          + ReturnFromHomeBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Money earned draw bets & "          + ReturnFromDrawBets[0].ToString("F3") + " & "          + ReturnFromDrawBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Money earned away bets & "          + ReturnFromAwayBets[0].ToString("F3") + " & "          + ReturnFromAwayBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg odds on bets & "                + AvgOddsOnBets[0].ToString("F3") + " & "               + AvgOddsOnBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg odds on home bets & "           + AvgOddsOnHomeBets[0].ToString("F3") + " & "           + AvgOddsOnHomeBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg odds on draw bets & "           + AvgOddsOnDrawBets[0].ToString("F3") + " & "           + AvgOddsOnDrawBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Avg odds on away bets & "           + AvgOddsOnAwayBets[0].ToString("F3") + " & "           + AvgOddsOnAwayBets[1].ToString("F3") + "\\\\ \n";
            tableValues += "Number of bets where  p*o < 1 &"    + NumberOfBetsNegExpReturn[0].ToString("F3") + " & "    + NumberOfBetsNegExpReturn[1].ToString("F3") + "\\\\ \n";
            tableValues += "Number of won bets where  p*o < 1 &"+ NumberOfWonBetsNegExpReturn[0].ToString("F3") + " & " + NumberOfWonBetsNegExpReturn[1].ToString("F3") + "\\\\ \n"; 
            tableValues += "Profit where p*o < 1 &"             + ReturnFromBetsNegExpRet[0].ToString("F3") + " & "     + ReturnFromBetsNegExpRet[1].ToString("F3") + "\\\\ \n";
            tableValues += "Number of bets o > 5 & "            + NumberOfBetsOddsGTFive[0].ToString("F3") + " & "      + NumberOfBetsOddsGTFive[1].ToString("F3") + "\\\\ \n";
            tableValues += "Number of won bets o > 5 & "        + NumberOfWonBetsOddsGTFive[0].ToString("F3") + " & "   + NumberOfWonBetsOddsGTFive[1].ToString("F3") + "\\\\ \n";
            tableValues += "Profit where o > 5 & "              + ReturnFromBetsOddsGTFive[0].ToString("F3") + " & "    + ReturnFromBetsOddsGTFive[1].ToString("F3") + "\\\\ \n";
            return tableValues;
        }
    }
}