﻿using FootballPredictions.Mappings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace FootballPredictions.Evaluation.Betting.Plots
{
    /// <summary>
    /// Write all the bets to file.
    /// </summary>
    public class TableOfBets : BettingPlot
    {
        public DateTime StartDate { get; set; }

        public override void Plot()
        {
            var prefix =   "\\begin{table}% \n"
                         + "\\centering \n"
                         + "\\caption{Table of bet outcomes} \n"
                         + "\\label{} \n"
                         + "\\begin{tabular}{ | c | c | c | c | c c c | c c | c |} \n"
                         + "\\hline \n"
                         + " & & & & \\multicolumn{3}{c}{Probability} & \\multicolumn{2}{c}{Odds} & \\\\ \n"
                         + "Match & Date & Outcome & Bet & Home & Draw & Away & Odds & Prob & Profit \\\\ \\hline \n"
                         + "";

            var suffix =   "\\hline \n"
                         + "\\end{tabular} \n"
                         + "\\end{table} \n";

            var tableValues = "";
            var output = "";
            var matchCount = 0;

            using (var session = NHibernateConfig.OpenSession())
            {
                foreach (var b in Bets)
                {
                    if (b.Match.DateAndTime.CompareTo(StartDate) == -1) continue;

                    matchCount++;
                    var homeTeam = session.Get<Team>(b.Match.HomeTeam.TeamId).Name.Trim('\'').Split(' ');
                    var awayTeam = session.Get<Team>(b.Match.AwayTeam.TeamId).Name.Trim('\'').Split(' ');

                    var teamNames = homeTeam.Length > 1 ? homeTeam[0].Substring(0, 3) + homeTeam[1].Substring(0, 1) : homeTeam[0].Substring(0, Math.Min(4,homeTeam[0].Length));
                    teamNames += awayTeam.Length > 1 ? " vs " + awayTeam[0].Substring(0, 3) + awayTeam[1].Substring(0, 1) : " vs " + awayTeam[0].Substring(0, Math.Min(4, awayTeam[0].Length));

                    var date = "" + b.Match.DateAndTime.Day + "/" + b.Match.DateAndTime.Month;

                    var outcome = b.Match.HomeGoals + "-" + b.Match.AwayGoals + "(" + b.Match.Result + ")";

                    var result = session.Get<Result>(b.Prediction.Result.ResultId);
                    var prediction = result.HomeGoals + "-" + result.AwayGoals;

                    var bet = 'N';
                    var probability = 0.0;
                    var odds = 0.0;
                    var profit = 0.0; 
                    
                    if (b.BetsPlaced[0] > 0)
                    {
                        bet = 'H';
                        probability = b.Prediction.HomeProbability;
                        odds = b.Odds.HomeOdds;
                        if (b.Match.Result == bet)
                        {
                            profit = b.BetsPlaced[0] * odds;
                        }
                        else
                        {
                            profit = -b.BetsPlaced[0];
                        }                    
                    }

                    if (b.BetsPlaced[1] > 0)
                    {
                        bet = 'D';
                        probability = b.Prediction.DrawProbability;
                        odds = b.Odds.DrawOdds;
                        if (b.Match.Result == bet)
                        {
                            profit = b.BetsPlaced[1] * odds;
                        }
                        else
                        {
                            profit = -b.BetsPlaced[1];
                        }
                    }
                    else if (b.BetsPlaced[2] > 0)
                    {
                        bet = 'A';
                        probability = b.Prediction.AwayProbability;
                        odds = b.Odds.AwayOdds;
                        if (b.Match.Result == bet)
                        {
                            profit = b.BetsPlaced[2] * odds;
                        }
                        else
                        {
                            profit = -b.BetsPlaced[2];
                        }
                    }
                    if (bet == b.Match.Result && bet != 'N')
                        tableValues += teamNames + "&" + date + "&" + outcome + "&" + "{\\color{green}" + bet + "}&" + b.Prediction.HomeProbability.ToString("F2") + "&" + b.Prediction.DrawProbability.ToString("F2") + "&" + b.Prediction.AwayProbability.ToString("F2") + "&" + odds + "&" + (1.0/odds).ToString("F2") + "&{\\color{green}" + profit + "}\\\\ \n";
                    else if (bet != 'N')
                        tableValues += teamNames + "&" + date + "&" + outcome + "&" + "{\\color{red}" + bet + "}&" + b.Prediction.HomeProbability.ToString("F2") + "&" + b.Prediction.DrawProbability.ToString("F2") + "&" + b.Prediction.AwayProbability.ToString("F2") + "&" + odds + "&" + (1.0 / odds).ToString("F2") + "&{\\color{red}" + profit + "}\\\\ \n";
                    else
                        tableValues += teamNames + "&" + date + "&" + outcome + "&" + bet + "&" + b.Prediction.HomeProbability.ToString("F2") + "&" + b.Prediction.DrawProbability.ToString("F2") + "&" + b.Prediction.AwayProbability.ToString("F2") + "&" + 0 + "&" + 0 + "&" + profit + "\\\\ \n";

                    if (matchCount == 35 || b == Bets.Last())
                    {
                        output += prefix + tableValues + suffix;
                        tableValues = "";
                        matchCount = 0;
                    }
                }
            }



            using(var file = new System.IO.StreamWriter(Constants.TEXT_FILES + "JAGS/Betting/tableOfBets_"+Stage+"_"+SimulationName+"_"+BettingStrategy+".tex"))
            {
                file.Write(output);
            }
        }
    }
}