﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;
using FootballPredictions.Mappings;
using SharpJags.Jags;

namespace FootballPredictions.JAGS.DixonColesStatic
{
    /// <summary>
    /// Static version of the Dixon Coles simualtion.
    /// </summary>
    public class DixonColesStaticPredictiveModel : PredictiveModel
    {

        public DixonColesStaticPredictiveModel()
        {
            AssignVariables();
        }

        public override void AssignVariables()
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {
                    var matches = from g in session.Query<Match>().ToList()
                                  where g.Stage.StageId == Constants.STAGE_ID
                                  select g;

                    foreach (var match in matches)
                    {
                        if (!Teams.Contains(match.HomeTeam))
                        {
                            Teams.Add(match.HomeTeam);
                        }
                        if (!Teams.Contains(match.AwayTeam))
                        {
                            Teams.Add(match.AwayTeam);
                        }
                    }

                    matches = matches.OrderBy(m => m.DateAndTime);

                    foreach (var match in matches)
                    {
                        Matches.Add(match);
                    }

                    NoTeams = Teams.Count();
                    NoMatches = matches.Count();
                    NoRounds = (int)(((double)NoMatches / NoTeams) * 2);
                }
            }
        }

        public override JagsData GetSeasonSpecificData()
        {
            // Initialize the matrices
            TeamMx = new double[NoMatches, 2];
            GoalMx = new double[NoMatches, 2];
            RoundMx = new double[NoMatches, 2];
            DayMx = new double[NoTeams, NoRounds];

            var teamMatchesPlayed = new double[NoTeams];
            var matchidx = 0;

            // Fill the matrices here
            foreach (var match in Matches)
            {
                var homeTeamIdx = Teams.IndexOf(match.HomeTeam);
                var awayTeamIdx = Teams.IndexOf(match.AwayTeam);

                teamMatchesPlayed[homeTeamIdx]++;
                teamMatchesPlayed[awayTeamIdx]++;

                TeamMx[matchidx, 0] = homeTeamIdx + 1;
                TeamMx[matchidx, 1] = awayTeamIdx + 1;

                RoundMx[matchidx, 0] = teamMatchesPlayed[homeTeamIdx];
                RoundMx[matchidx, 1] = teamMatchesPlayed[awayTeamIdx];

                if (matchidx > Constants.SIZE_OF_TRAINING_SET)
                {
                    GoalMx[matchidx, 0] = Double.NaN;
                    GoalMx[matchidx, 1] = Double.NaN;
                }
                else
                {
                    GoalMx[matchidx, 0] = match.HomeGoals;
                    GoalMx[matchidx, 1] = match.AwayGoals;
                }
                matchidx++;
            }

            // Returns the matrices
            return new JagsData
            {
                { "team", TeamMx },
                { "schedule", TeamMx },
                { "goalsScored", GoalMx },
                { "noGames", NoMatches },
                { "noRounds", NoRounds},
                { "noTeams", NoTeams },
            };
        }

        public override JagsData GetMatchSpecificJagsData(int matchNumber)
        {
            throw new NotImplementedException();
        }

        public override JagsData GetModelPriors()
        {
            return null;
        }
    }
}