﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using MathNet.Numerics.LinearAlgebra.Double;
using NHibernate.Linq;
using SharpJags.Collections;
using SharpJags.Jags;
using SharpJags.Parsing;
using SharpJags.Math;

using FootballPredictions.Mappings;

namespace FootballPredictions.JAGS
{
    /// <summary>
    /// Super class for prediction models. Must be extended by all the prediction models.
    /// </summary>
    public abstract class PredictiveModel
    {
        public List<Team> Teams = new List<Team>();
        public List<Match> Matches = new List<Match>();
        public int NoTeams { get; set; }
        public int NoMatches { get; set; }
        public int NoRounds { get; set; }

        protected Matrix<double> TeamMx { get; set; }
        protected Matrix<double> GoalMx { get; set; }
        // Timeslices i Thomas og Damian sin kode...
        protected Matrix<double> RoundMx { get; set; }
        protected Matrix<double> DayMx { get; set; }

        protected long StartTime { get; set; }

        public abstract void AssignVariables();
        public abstract JagsData GetSeasonSpecificData();
        public abstract JagsData GetMatchSpecificJagsData(int matchNumber);
        public abstract JagsData GetModelPriors();
    }
}