﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Mapping.ByCode;
using NHibernate.Mapping.ByCode.Conformist;

namespace FootballPredictions.Mappings
{
    /// <summary>
    /// The Match object.
    /// </summary>
    public class Match
    {
        public virtual int MatchId { get; set; }
        public virtual Stage Stage { get; set; }
        public virtual Team HomeTeam { get; set; }
        public virtual Team AwayTeam { get; set; }
        public virtual DateTime DateAndTime { get; set; }
        public virtual int HomeGoals { get; set; }
        public virtual int AwayGoals { get; set; }
        // Either H, D or A
        public virtual char Result { get; set; }
    }
    /// <summary>
    /// Mapping for the database.
    /// </summary>
    public class MatchMap : ClassMapping<Match>
    {
        public MatchMap()
        {
            Id(x => x.MatchId);

            Property(x => x.DateAndTime);
            Property(x => x.HomeGoals);
            Property(x => x.AwayGoals);
            Property(x => x.Result);

            ManyToOne(x => x.Stage, m =>
                {
                    m.Cascade(Cascade.DeleteOrphans);
                    m.NotNullable(true);
                });
            ManyToOne(x => x.HomeTeam, m =>
                {
                    m.Cascade(Cascade.Remove);
                    m.NotNullable(true);
                });
            ManyToOne(x => x.AwayTeam, m =>
                {
                    m.Cascade(Cascade.Remove);
                    m.NotNullable(true);
                });
        }
    }
}