﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Mapping.ByCode;
using NHibernate.Mapping.ByCode.Conformist;

namespace FootballPredictions.Mappings
{
    /// <summary>
    /// The relations between a Player and a Match, showing the Player's contributions
    /// in the Match.
    /// </summary>
    public class PlayerMatch
    {
        public virtual int PlayerMatchId { get; set; }

        public virtual Player Player { get; set; }
        public virtual TeamMatch TeamMatch { get; set; }

        // General Match contribution info
        public virtual bool GameStarted { get; set; }
        public virtual int MinsPlayed { get; set; }
        public virtual bool ManOfTheMatch { get; set; }
        public virtual double Rating { get; set; }
        public virtual string Position { get; set; }
        public virtual int FormationPlace { get; set; }

        // Goalkeeper specific contributions
        public virtual int Saves { get; set; }
        public virtual int GoodHighClaim { get; set; }
        public virtual int PenaltySave { get; set; }

        // Goals
        public virtual int Goals { get; set; }
        public virtual int OwnGoals { get; set; }
        public virtual int OwnGoalAccrued { get; set; }
        public virtual int AttPenGoal { get; set; }

        // Attack contributions
        public virtual int Touches { get; set; }
        public virtual int UnsuccessfulTouch { get; set; }
        public virtual int GoalAssist { get; set; }        
        public virtual int TotalPass { get; set; }
        public virtual int AccuratePass { get; set; }
        public virtual int TotalThroughBall { get; set; }
        public virtual int AccurateThroughBall { get; set; }
        public virtual int TotalFinalThirdPasses { get; set; }
        public virtual int SuccessfulFinalThirdPasses { get; set; }
        public virtual int TotalAttAssist { get; set; }
        public virtual int OfftargetAttAssist { get; set; }
        public virtual int BigChanceCreated { get; set; }
        public virtual int TotalOffside { get; set; }
        public virtual int Dispossessed { get; set; }
        
        // Scoring attempts
        public virtual int TotalScoringAtt { get; set; }
        public virtual int BlockedScoringAtt { get; set; }
        public virtual int OntargetScoringAtt { get; set; }
        public virtual int ShotOffTarget { get; set; }
        public virtual int PostScoringAtt { get; set; }
        public virtual int BigChanceMissed { get; set; }

        // Defensive contributions
        public virtual int TotalTackle { get; set; }
        public virtual int LastManTackle { get; set; }
        public virtual int TotalClearance { get; set; }
        public virtual int EffectiveClearance { get; set; }
        public virtual int SixYardBlock { get; set; }
        public virtual int ClearanceOffLine { get; set; }
        public virtual int ErrorLeadToGoal { get; set; }
        public virtual int ErrorLeadToShot { get; set; }
        public virtual int Interception { get; set; }
        public virtual int InterceptionWon { get; set; }
        public virtual int BallRecovery { get; set; }

        // Set plays and fouls
        public virtual int WonCorners { get; set; }
        public virtual int TotalThrows { get; set; }
        public virtual int PenaltyConceded { get; set; }
        public virtual int Fouls { get; set; }
        public virtual int YellowCard { get; set; }
        public virtual int RedCard { get; set; }
        
        // Duelling
        public virtual int AerialWon { get; set; }
        public virtual int AerialLost { get; set; }
        public virtual int DuelWon { get; set; }
        public virtual int DuelLost { get; set; }
        public virtual int WonContest { get; set; }
        public virtual int TotalContest { get; set; }
    }
    /// <summary>
    /// Database mapping.
    /// </summary>
    public class PlayerMatchMap : ClassMapping<PlayerMatch>
    {
        public PlayerMatchMap()
        {
            Id(x => x.PlayerMatchId, m =>
            {
                m.Generator(Generators.Identity);
            });

            // General match contribution
            Property(x => x.GameStarted);
            Property(x => x.MinsPlayed);
            Property(x => x.ManOfTheMatch);
            Property(x => x.Rating);
            Property(x => x.Position);
            Property(x => x.FormationPlace);

            // Goalkeeper specific contributions
            Property(x => x.Saves);
            Property(x => x.GoodHighClaim);
            Property(x => x.PenaltySave);

            // Goals
            Property(x => x.Goals);
            Property(x => x.OwnGoals);
            Property(x => x.OwnGoalAccrued);
            Property(x => x.AttPenGoal);
            
            // Attack contributions
            Property(x => x.Touches);
            Property(x => x.UnsuccessfulTouch);
            Property(x => x.GoalAssist);
            Property(x => x.TotalPass); 
            Property(x => x.AccuratePass);
            Property(x => x.TotalThroughBall);
            Property(x => x.AccurateThroughBall);
            Property(x => x.TotalFinalThirdPasses);
            Property(x => x.SuccessfulFinalThirdPasses);
            Property(x => x.TotalAttAssist);
            Property(x => x.OfftargetAttAssist);
            Property(x => x.BigChanceCreated);
            Property(x => x.TotalOffside);
            Property(x => x.Dispossessed);

            // Scoring attempts
            Property(x => x.TotalScoringAtt);
            Property(x => x.BlockedScoringAtt);
            Property(x => x.OntargetScoringAtt);
            Property(x => x.ShotOffTarget);
            Property(x => x.PostScoringAtt);
            Property(x => x.BigChanceMissed);

            // Defensive contributions
            Property(x => x.TotalTackle);
            Property(x => x.LastManTackle);
            Property(x => x.TotalClearance);
            Property(x => x.EffectiveClearance);
            Property(x => x.SixYardBlock);
            Property(x => x.ClearanceOffLine);
            Property(x => x.ErrorLeadToGoal);
            Property(x => x.ErrorLeadToShot);
            Property(x => x.Interception);
            Property(x => x.InterceptionWon);
            Property(x => x.BallRecovery);

            // Set plays and fouls
            Property(x => x.WonCorners);
            Property(x => x.TotalThrows);
            Property(x => x.PenaltyConceded); 
            Property(x => x.Fouls);
            Property(x => x.YellowCard);
            Property(x => x.RedCard);

            // Duelling
            Property(x => x.AerialWon);
            Property(x => x.AerialLost);
            Property(x => x.DuelWon);
            Property(x => x.DuelLost);
            Property(x => x.WonContest);
            Property(x => x.TotalContest);

            ManyToOne(x => x.Player, m =>
                {
                    m.Cascade(Cascade.None);
                    m.NotNullable(true);
                });
            ManyToOne(x => x.TeamMatch, m =>
                {
                    m.Cascade(Cascade.None);
                    m.NotNullable(true);
                });
        }
    }
}