﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Mapping.ByCode;
using NHibernate.Mapping.ByCode.Conformist;

namespace FootballPredictions.Mappings
{
    /// <summary>
    /// Prediction object, with the predictions generated during a simulation.
    /// </summary>
    public class Prediction
    {
        public virtual int PredictionId { get; set; }

        public virtual Simulation Simulation { get; set; }
        public virtual Match Match { get; set; }

        public virtual TeamStrength HomeStrength { get; set; }
        public virtual TeamStrength AwayStrength { get; set; }

        public virtual Result Result { get; set; }

        public virtual double HomeProbability { get; set; }
        public virtual double DrawProbability { get; set; }
        public virtual double AwayProbability { get; set; }
    }
    /// <summary>
    /// Mapping for the database.
    /// </summary>
    public class PredictionMap : ClassMapping<Prediction>
    {
        public PredictionMap()
        {
            Id(x => x.PredictionId, m =>
            {
                m.Generator(Generators.Identity);
            });

            Property(x => x.HomeProbability);
            Property(x => x.DrawProbability);
            Property(x => x.AwayProbability);

            ManyToOne(x => x.Simulation, m =>
            {
                m.Cascade(Cascade.None);
                m.NotNullable(true);
            });

            ManyToOne(x => x.Match, m =>
                {
                    m.Cascade(Cascade.None);
                    m.NotNullable(true);
                });

            ManyToOne(x => x.HomeStrength, m =>
            {
                m.Cascade(Cascade.None);
                m.NotNullable(true);
            });

            ManyToOne(x => x.AwayStrength, m =>
            {
                m.Cascade(Cascade.None);
                m.NotNullable(true);
            });

            ManyToOne(x => x.Result, m =>
            {
                m.Cascade(Cascade.None);
                m.NotNullable(true);
            });
        }
    }
}