﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Mapping.ByCode;
using NHibernate.Mapping.ByCode.Conformist;

namespace FootballPredictions.Mappings
{
    /// <summary>
    /// Statistics of how a team performed in a match.
    /// </summary>
    public class TeamMatch
    {
        public virtual int TeamMatchId { get; set; }
        
        public virtual Team Team { get; set; }
        public virtual Match Match { get; set; }

        // General match attributes
        public virtual double Rating { get; set; }
        public virtual double PossessionPercentage { get; set; }

        // Goals
        public virtual int Goals { get; set; }
        public virtual int OwnGoals { get; set; }
        public virtual int OwnGoalAccrued { get; set; }     //Own goal by opposition
        public virtual int AttPenGoal { get; set; }

        // Attack contributions
        public virtual int Touches { get; set; }
        public virtual int UnsuccessfulTouch { get; set; }
        public virtual int GoalAssist { get; set; }
        public virtual int TotalPass { get; set; }
        public virtual int AccuratePass { get; set; }
        public virtual int TotalThroughBall { get; set; }
        public virtual int AccurateThroughBall { get; set; }
        public virtual int TotalFinalThirdPasses { get; set; }
        public virtual int SuccessfulFinalThirdPasses { get; set; }
        public virtual int TotalAttAssist { get; set; }     //Key passes (leads to a scoring attempt)
        public virtual int OfftargetAttAssist { get; set; } //Key passes where shot is off target?
        public virtual int BigChanceCreated { get; set; }
        public virtual int TotalOffside { get; set; }
        public virtual int Dipossessed { get; set; }

        // Scoring attempts
        public virtual int TotalScoringAtt { get; set; }    //Total shots
        public virtual int BlockedScoringAtt { get; set; }  //Shots blocked
        public virtual int OntargetScoringAtt { get; set; } //Shots on goal
        public virtual int ShotOffTarget { get; set; }      //Shots off target
        public virtual int PostScoringAtt { get; set; }     //Hit woodwork
        public virtual int BigChanceMissed { get; set; }

        // Defensive contributions
        public virtual int TotalTackle { get; set; }
        public virtual int LastManTackle { get; set; }
        public virtual int TotalClearance { get; set; }
        public virtual int EffectiveClearance { get; set; }
        public virtual int SixYardBlock { get; set; }
        public virtual int ClearanceOffLine { get; set; }
        public virtual int ErrorLeadToGoal { get; set; }
        public virtual int ErrorLeadToShot { get; set; }
        public virtual int Interception { get; set; }
        public virtual int InterceptionWon { get; set; }
        public virtual int BallRecovery { get; set; }

        // Set Plays and fouls
        public virtual int CornerTaken { get; set; }
        public virtual int TotalThrows { get; set; }
        public virtual int PenaltyConceded { get; set; }
        public virtual int FkFoulWon { get; set; }          //Free kicks won
        public virtual int FkFoulLost { get; set; }         //Total fouls commited by Team
        public virtual int YellowCard { get; set; }
        public virtual int RedCard { get; set; }

        // Duelling
        public virtual int AerialLost { get; set; }
        public virtual int AerialWon { get; set; }
        public virtual int DuelLost { get; set; }
        public virtual int DuelWon { get; set; }
        public virtual int WonContest { get; set; }         // Dribbles won
        public virtual int TotalContest { get; set; }

        // Penalties
        public virtual int PenaltySave { get; set; }
    }
    /// <summary>
    /// Mapping for the database.
    /// </summary>
    public class TeamMatchMap : ClassMapping<TeamMatch>
    {
        public TeamMatchMap()
        {
            Id(x => x.TeamMatchId, m =>
            {
                m.Generator(Generators.Identity);
            });

            // General match attributes
            Property(x => x.Rating);
            Property(x => x.PossessionPercentage);

            //Goals
            Property(x => x.Goals);
            Property(x => x.OwnGoals);
            Property(x => x.OwnGoalAccrued);
            Property(x => x.AttPenGoal);
            
            // Attack contributions
            Property(x => x.Touches);
            Property(x => x.UnsuccessfulTouch);
            Property(x => x.GoalAssist);
            Property(x => x.TotalPass);
            Property(x => x.AccuratePass);
            Property(x => x.TotalThroughBall);
            Property(x => x.AccurateThroughBall);
            Property(x => x.TotalFinalThirdPasses);
            Property(x => x.SuccessfulFinalThirdPasses);
            Property(x => x.TotalAttAssist);
            Property(x => x.OfftargetAttAssist);
            Property(x => x.BigChanceCreated);
            Property(x => x.TotalOffside);
            Property(x => x.Dipossessed);

            // Scoring attempts
            Property(x => x.TotalScoringAtt);
            Property(x => x.BlockedScoringAtt);
            Property(x => x.OntargetScoringAtt);
            Property(x => x.ShotOffTarget);
            Property(x => x.PostScoringAtt);
            Property(x => x.BigChanceMissed);

            // Defensive contributions
            Property(x => x.TotalTackle);
            Property(x => x.LastManTackle);
            Property(x => x.TotalClearance);
            Property(x => x.EffectiveClearance);
            Property(x => x.SixYardBlock);
            Property(x => x.ClearanceOffLine);
            Property(x => x.ErrorLeadToGoal);
            Property(x => x.ErrorLeadToShot);
            Property(x => x.Interception);
            Property(x => x.InterceptionWon);
            Property(x => x.BallRecovery);

            // Set plays and fouls
            Property(x => x.CornerTaken);
            Property(x => x.TotalThrows);
            Property(x => x.PenaltyConceded);
            Property(x => x.FkFoulLost);
            Property(x => x.FkFoulWon);
            Property(x => x.YellowCard);
            Property(x => x.RedCard);

            // Duelling
            Property(x => x.WonContest);
            Property(x => x.TotalContest);
            Property(x => x.AerialLost);
            Property(x => x.AerialWon);
            Property(x => x.DuelLost);
            Property(x => x.DuelWon);

            // Penalties
            Property(x => x.PenaltySave);
            
            // Relations to Team and Match
            ManyToOne(x => x.Team, m =>
                {
                    m.Cascade(Cascade.None);
                    m.NotNullable(true);
                });
            ManyToOne(x => x.Match, m =>
                {
                    m.Cascade(Cascade.None);
                    m.NotNullable(true);
                });
        }
    }
}