﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using System.Data;
using NHibernate;
using NHibernate.Cfg;
using NHibernate.Cfg.MappingSchema;
using NHibernate.Dialect;
using NHibernate.Driver;
using NHibernate.Mapping.ByCode;
using NHibernate.Tool.hbm2ddl;

using FootballPredictions;
using FootballPredictions.DataHandling;
using FootballPredictions.Mappings;
using FootballPredictions.DataHandling.PopulateDatabase;

namespace FootballPredictions
{
    /// <summary>
    /// Required for setting up the database
    /// </summary>
    public class NHibernateConfig
    {
        public NHibernateConfig()
        {
            Setup();
            if (Constants.ADD_NEW_TABLE)
            {
                CreateDatabaseSchema();
            }
            if (Constants.UPDATE_DATABASE)
            {
                if (ValidateSchema())
                {
                    var PopulateDatabase = new PopulateDatabase();
                }
            }
        }

        protected static Configuration NHConfiguration;
        protected static ISessionFactory SessionFactory;

        private static Configuration ConfigureNHibernate()
        {
            var configure = new Configuration();

            configure.DataBaseIntegration(db =>
                {
                    db.Dialect<MsSqlCeDialect>();
                    db.Driver<SqlServerCeDriver>();
                    db.KeywordsAutoImport = Hbm2DDLKeyWords.AutoQuote;
                    db.IsolationLevel = IsolationLevel.ReadCommitted;

                    db.ConnectionString = "Data Source=" + Constants.DATABASE;
                    db.Timeout = 10;

                    // Enabled for testing
                    db.LogFormattedSql = true;
                    db.LogSqlInConsole = true;
                    db.AutoCommentSql = true;
                });
            
            return configure;
        }

        // Mapping of the objects to the database
        protected static HbmMapping GetMappings()
        {
            ModelMapper mapper = new ModelMapper();
            // Should make this automatic, such that there is no need for manual adding of new classes.
            mapper.AddMapping<PlayerMap>();
            mapper.AddMapping<TeamMap>();
            mapper.AddMapping<MatchMap>();
            mapper.AddMapping<StageMap>();
            mapper.AddMapping<TeamMatchMap>();
            mapper.AddMapping<PlayerMatchMap>();
            mapper.AddMapping<ProbableLineUpMap>();
            mapper.AddMapping<ResultMap>();
            mapper.AddMapping<TeamStrengthMap>();
            mapper.AddMapping<SimulationMap>();
            mapper.AddMapping<PredictionMap>();
            mapper.AddMapping<BookmakerMap>();
            mapper.AddMapping<OddsMap>();

            HbmMapping mapping = mapper.CompileMappingFor(new[] 
            {
                // This should be automatic too.
                typeof(Player),
                typeof(Team),
                typeof(Match),
                typeof(Stage),
                typeof(TeamMatch),
                typeof(PlayerMatch),
                typeof(ProbableLineUp),
                typeof(Result),
                typeof(TeamStrength),
                typeof(Simulation),
                typeof(Prediction),
                typeof(Bookmaker),
                typeof(Odds),
            });

            return mapping;
        }

        public static void Setup()
        {
            NHConfiguration = ConfigureNHibernate();
            HbmMapping mapping = GetMappings();
            NHConfiguration.AddDeserializedMapping(mapping, null);
            SchemaMetadataUpdater.QuoteTableAndColumns(NHConfiguration);
            SessionFactory = NHConfiguration.BuildSessionFactory();
        }

        protected static void CreateDatabaseSchema()
        {
            new SchemaExport(NHConfiguration).Create(false, true);
        }

        protected static bool ValidateSchema()
        {
            try
            {
                SchemaValidator schemaValidator = new SchemaValidator(NHConfiguration);
                schemaValidator.Validate();
                return true;
            }
            catch (HibernateException e)
            {
                Console.WriteLine(e.Message);
                return false;
            }
        }

        public static ISession OpenSession()
        {
            var session = SessionFactory.OpenSession();
            return session;
        }
    }
}