<?php
//This section deals with getting information from a single match.
//
// Note! : WhoScored.com only remembers the last 6 games played, the last 6 home games and last 6 away games, so 12 maximum (unless too early in the season).
// We therefore need the legacy-matches file for all matches that are not listed in whoscored, but still exist on their site (e.g. Man U -Everton, first game of the season,
// can still be found, but we need to store this as it will not be contained in the code snippet used for getting table and match info).

/*

	ORDER
	
	1. Load fixtureInfo.csv (done globally)
	2. For each team, get matchlist from legacy-matches. (done in Team constructor)
	3. For each match that isnt in fixtureInfo.csv, load that whoscored page and do preg_match to extract detailed match info. (placed in $NewMatchInfoGathered)
	4. Save new match info to fixtureInfo.csv.
	

*/

ini_set('pcre.backtrack_limit', '1048576');
include '../attributes.php';
error_reporting(-1);

class Team {
	
	public $name = "";
	public $fixtureIDList = array();
	public $fixtureInfoList = array();
	
	function __construct($teamName){
		$this->name = $teamName;
		$this->fixtureIDList = getFixtureListForTeam($this->name);
		for($i = 0; $i < sizeof($this->fixtureIDList); $i++){
			$this->fixtureInfoList[$i] = getWhoScoredMatchInfo($this->fixtureIDList[$i]);
		}
	}
	
	public function printTeamInfo(){
		echo $this->name.": ";
		foreach($this->fixtureIDList as $fixture){
			echo $fixture.", ";
		}
		echo "<br>";
	}
}

class Fixture{
	public $infoString = "";
	public $infoArray = array();

}

/**
Global vars
**/
$teamNameArray = getAllTeamNames();											// Array containing names of each team in the league, inside ' '.
$teamObjectArray = array();													// Array of Team objects. 
$timer = time();															// Sets starting time to know when to stop computing for safe completion of crawler.
$maximumRunTime = 20;
$currentFixturesCrawled = 0;												// Extra set of limits to ensure safe completion of crawler.
/**
**/

//Main method for this file, creates new team-objects. In each constructor, whoscored.com is crawled for new matches, the matches are added to objects as well as new ones are written
//written to file to . 
function GetNewMatchObjects(){
	global $teamNameArray, $teamObjectArray, $timer, $maximumRunTime, $currentFixturesCrawled, $csv_fixtureInfo;
	
	echo "Getting New Match objects...</br>";
	for($i = 0; $i < sizeof($teamNameArray); $i++){
		$teamObjectArray[$i] = new Team($teamNameArray[$i]);
		if(takenTooLongTime($timer, $maximumRunTime) || $currentFixturesCrawled > 0){break;}
	}
	echo "Done Retrieving match objects! </br>";
}

function appendArrayToArray($toArray, $fromArray){
	$lengthTo = sizeof($toArray);
	$lengthFrom = sizeof($fromArray);
	$j = 0;
	for($i = $lengthTo; $i < $lengthTo + $lengthFrom; $i++){
		$toArray[$i] = $fromArray[$j];
		$j++;
	}
	return $toArray;
}

//Loading pages takes a long time, if we don't take care, the program might cease before it is done. This method makes for a more a safe running, by halting if exceeded maxTime seconds.
function takenTooLongTime($startTime, $maxTime){
	if(time() - $startTime > $maxTime) {return true;}	
	return false;
}

/*
	read from matches.csv, and returns an array containing all the team names.
*/
function getAllTeamNames(){
	global $csv_matches;

	$handle = @fopen($csv_matches, "r");
	if ($handle) {
		$arrayOfTeamNames = array();
		$teamCounter = 0;
		while (($buffer = fgets($handle, 4096)) !== false) { 	//read one line at a time
			$tempArray = explode(",", $buffer); 			//First two elements should be teamId and name
			$arrayOfTeamNames[$teamCounter] = $tempArray[1];
			$teamCounter++;
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
		//echo "Done, Teams: ".printArray($arrayOfTeamNames);
		return $arrayOfTeamNames;
	}

}

/*
	Using legacy-matches.csv, uses $teamname to find correct line in file, and returns $arrayOfFixtures, -> Array( [1]=>612240, [2]=>615008 .... etc. )
	The array then holds all the fixture-IDs for each match the chosen team has played.
*/
function getFixtureListForTeam($teamName){
	global $csv_legacyMatches;
	$handle = @fopen($csv_legacyMatches, "r");
	$foundTeam = false;
	if ($handle) {
		$arrayOfFixtures = array();
		while (($buffer = fgets($handle, 4096)) !== false) { 	//read one line at a time
			$tempArray = explode(",", $buffer); 			//First two elements should be teamId and name
			if($tempArray[1] == $teamName){
				$foundTeam = true;
				for($i = 2; $i < sizeof($tempArray); $i++){
					$arrayOfFixtures[$i-2] = trim($tempArray[$i], " \n");
				}
				
			}
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
		//if($foundTeam) echo "Done, TeamName: $teamName, fixtures: ".printArray($arrayOfFixtures);
		return $arrayOfFixtures;
	}

}


function getTeamIDFromName($teamName){
	global $csv_matches;
	$handle = @fopen($csv_matches, "r");
	$foundTeam = false;
	if ($handle) {
		$ID = 0;
		while (($buffer = fgets($handle, 4096)) !== false) { 	//read one line at a time
			$tempArray = explode(",", $buffer); 			//First two elements should be teamId and name
			if($tempArray[1] == $teamName){
				$foundTeam = true;
				$ID = $tempArray[0];
				
			}
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
		//if($foundTeam) echo "Done, TeamName: $teamName, fixtures: ".printArray($arrayOfFixtures);
		return $ID;
	}

}

function getTeamNameFromID($teamID){
	global $csv_matches;
	$handle = @fopen($csv_matches, "r");
	$foundTeam = false;
	if ($handle) {
		$teamName = "";
		while (($buffer = fgets($handle, 4096)) !== false) { 	//read one line at a time
			$tempArray = explode(",", $buffer); 			//First two elements should be teamId and name
			if($tempArray[0] == $teamID){
				$foundTeam = true;
				$teamName = $tempArray[1];
				
			}
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
		//if($foundTeam) echo "Done, TeamName: $teamName, fixtures: ".printArray($arrayOfFixtures);
		return $teamName;
	}

}

//Assumes a 1D array
function printArray($array){
	$outputString = "";
	foreach ($array as $item){
		$outputString = "$outputString$item, ";
	}
	return substr($outputString, 0 , strlen($outputString)-2);
}

function print2DArray($array){
	$outputString = "";
	foreach ($array as $list){
		foreach ($list as $item){
			$outputString = "$outputString$item, ";
		}
		$outputString = "$outputString</br>";
	}
	return substr($outputString, 0 , strlen($outputString)-2);
}

/*
Crawls WhoScored.com for all new matches we do not have any info on. Player and team statistics are written to file.
*/
function getWhoScoredMatchInfo($matchID){
	global $currentFixturesCrawled, $matchInfoDir;
	if(file_exists($matchInfoDir.trim($matchID, " \n").".txt")) {return;} 					//Loading page takes a long time, no need if we already have the info in a .txt file.
	
	$probableLineups = getProbableLineups($matchID);

	$currentFixturesCrawled++;
	$target_url = "http://www.whoscored.com/Matches/".trim($matchID, " \n")."/LiveStatistics";
	//echo $target_url."</br>";
	$html = new simple_html_dom();
	$html->load_file($target_url);
	$subject = $html;
	
	$generalInfoPattern = "/var initialData[^;]*/";
	$noDetailedInfoAvailable = false;
	
	if(preg_match($generalInfoPattern, $subject, $matches))
	{
		$matches[sizeof($matches)] = $probableLineups;
		appendArrayToFile($matchInfoDir.trim($matchID, " \n").".txt", $matches);
	}
	else{
		echo "Either no match fiund for RegEx, or WhoScored does not seem to have detailed information on the match.</br>
		Check source code at <a href=\"http://www.whoscored.com/Matches/".$matchID."/LiveStatistics\">WhoScored.com</a></br>";
		$noDetailedInfoAvailable = true;
	}
	if($noDetailedInfoAvailable){
	}
	else{
	}
}

/*
Crawls WhoScored for their suggestion of probable lineups for a match.
*/
function getProbableLineups($matchID) {
	global $matchInfoDir, $currentFixturesCrawled;
	$currentFixturesCrawled++;
	$target_url = "http://www.whoscored.com/Matches/".trim($matchID, " \n")."/Preview";
	$html = new simple_html_dom();
	$html->load_file($target_url);

	$subject = $html;
	$lineUpPattern = "#href=\"/Players/[0-9]*/Show/.*?\"#";	//The 22 first matches of this pattern are the starting lineups
	//echo "Vi er her! ".$target_url."</br>";
	if(preg_match_all($lineUpPattern, $subject, $matches)) 
	{
		$returnString = "";
		$returnString.="home: ";
		for($i=0; $i<11; $i++) {
			if($i==10) {
				$returnString.=preg_replace("/[^0-9]/", "", $matches[0][$i]);
			}
			else {
				$returnString.=preg_replace("/[^0-9]/", "", $matches[0][$i]).",";
			}
		}
		$returnString.="\naway: ";
		for($j=11; $j<22; $j++) {
			if($j==21) {
				$returnString.=preg_replace("/[^0-9]/", "", $matches[0][$j]);
			}
			else {
				$returnString.=preg_replace("/[^0-9]/", "", $matches[0][$j]).",";
			}
		}
		$returnString.="\n";
		echo $returnString;
		return $returnString;
	}
	else {
		echo "No probable lineup matches found for RegEx ".$lineUpPattern.", or WhoScored does not seem to have detailed information on the match.</br>
		Check source code at <a href=\"http://www.whoscored.com/Matches/".$matchID."/Preview\">WhoScored.com</a></br>";
	}
}

function appendToArray($array, $value){
	$array[sizeof($array)] = $value;
	return $array;

}

function appendArrayToFile($fileName, $array){
	$fh = fopen($fileName, 'a') or die("can't open file");
	foreach ($array as $matchInfo){
		fwrite($fh, trim($matchInfo, "\n")."\n");
	}
	fclose($fh);
}

/*********************/
//GetNewMatchObjects();



?>